@@ -3,8 +3,8 @@
  * License.....: MIT
  */
 
-#ifndef _DICTSTAT_H
-#define _DICTSTAT_H
+#ifndef HC_DICTSTAT_H
+#define HC_DICTSTAT_H
 
 #include <stdio.h>
 #include <string.h>
@@ -18,7 +18,7 @@
 #define MAX_DICTSTAT 100000
 
 #define DICTSTAT_FILENAME "hashcat.dictstat2"
-#define DICTSTAT_VERSION  (0x6863646963743200 | 0x02)
+#define DICTSTAT_VERSION  (0x6863646963743200 | 0x03)
 
 int sort_by_dictstat (const void *s1, const void *s2);
 
@@ -29,4 +29,4 @@ int  dictstat_write   (hashcat_ctx_t *hashcat_ctx);
 u64  dictstat_find    (hashcat_ctx_t *hashcat_ctx, dictstat_t *d);
 void dictstat_append  (hashcat_ctx_t *hashcat_ctx, dictstat_t *d);
 
-#endif // _DICTSTAT_H
+#endif // HC_DICTSTAT_H
