@@ -3,8 +3,8 @@
  * License.....: MIT
  */
 
-#ifndef _CPU_OPENCL_EMU_H
-#define _CPU_OPENCL_EMU_H
+#ifndef HC_CPU_OPENCL_EMU_H
+#define HC_CPU_OPENCL_EMU_H
 
 #define DEVICE_TYPE    -1
 #define VENDOR_ID      -1
@@ -23,11 +23,8 @@ typedef struct digest
 } digest_t;
 #endif
 
-u32 atomic_dec (u32 *p);
-u32 atomic_inc (u32 *p);
-
 size_t get_global_id  (u32 dimindx __attribute__((unused)));
 size_t get_local_id   (u32 dimindx __attribute__((unused)));
 size_t get_local_size (u32 dimindx __attribute__((unused)));
 
-#endif // _CPU_OPENCL_EMU_H
+#endif // HC_CPU_OPENCL_EMU_H
