@@ -3,12 +3,12 @@
  * License.....: MIT
  */
 
-#ifndef _EXT_LZMA_H
+#ifndef HC_EXT_LZMA_H
+#define HC_EXT_LZMA_H
 
 #include <LzmaDec.h>
 #include <Lzma2Dec.h>
 
-#include "minizip/ioapi.h"
 #include "minizip/unzip.h"
 
 int hc_lzma1_decompress (const unsigned char *in, SizeT *in_len, unsigned char *out, SizeT *out_len, const char *props);
@@ -17,4 +17,4 @@ int hc_lzma2_decompress (const unsigned char *in, SizeT *in_len, unsigned char *
 void *hc_lzma_alloc (MAYBE_UNUSED ISzAllocPtr p, size_t size);
 void  hc_lzma_free  (MAYBE_UNUSED ISzAllocPtr p, void *address);
 
-#endif // _EXT_LZMA_H
+#endif // HC_EXT_LZMA_H
