@@ -3,6 +3,9 @@
  * License.....: MIT
  */
 
+#ifndef HC_FOLDER_H
+#define HC_FOLDER_H
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -34,3 +37,6 @@ int  folder_config_init    (hashcat_ctx_t *hashcat_ctx, MAYBE_UNUSED const char
 void folder_config_destroy (hashcat_ctx_t *hashcat_ctx);
 
 int hc_mkdir (const char *name, MAYBE_UNUSED const int mode);
+int hc_mkdir_rec (const char *path, MAYBE_UNUSED const int mode);
+
+#endif // HC_FOLDER_H
