@@ -3,8 +3,8 @@
  * License.....: MIT
  */
 
-#ifndef _INDUCT_H
-#define _INDUCT_H
+#ifndef HC_INDUCT_H
+#define HC_INDUCT_H
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -16,4 +16,4 @@ int  induct_ctx_init    (hashcat_ctx_t *hashcat_ctx);
 void induct_ctx_scan    (hashcat_ctx_t *hashcat_ctx);
 void induct_ctx_destroy (hashcat_ctx_t *hashcat_ctx);
 
-#endif // _INDUCT_H
+#endif // HC_INDUCT_H
