@@ -3,8 +3,8 @@
  * License.....: MIT
  */
 
-#ifndef _OUTFILE_CHECK_H
-#define _OUTFILE_CHECK_H
+#ifndef HC_OUTFILE_CHECK_H
+#define HC_OUTFILE_CHECK_H
 
 #include <unistd.h>
 #include <errno.h>
@@ -16,4 +16,4 @@ HC_API_CALL void *thread_outfile_remove (void *p);
 int  outcheck_ctx_init    (hashcat_ctx_t *hashcat_ctx);
 void outcheck_ctx_destroy (hashcat_ctx_t *hashcat_ctx);
 
-#endif // _OUTFILE_CHECK_H
+#endif // HC_OUTFILE_CHECK_H
