@@ -13,7 +13,11 @@
 
 #if defined (__APPLE__)
 #include "event.h"
-#elif defined (__FreeBSD__) || defined (__NetBSD__)
+#elif defined (__OpenBSD__)
+#define _OPENBSD_SOURCE
+#include <stdlib.h>
+#include <sys/sysctl.h>
+#elif defined (__FreeBSD__) || defined (__NetBSD__) || defined (__DragonFly__)
 #include <sys/param.h>
 #include <sys/sysctl.h>
 #endif
@@ -46,7 +50,7 @@ static int get_exec_path (char *exec_path, const size_t exec_path_sz)
 
   const size_t len = strlen (exec_path);
 
-  #elif defined (__FreeBSD__)
+  #elif defined (__FreeBSD__) || defined (__DragonFly__)
 
   int mib[4];
 
@@ -76,6 +80,41 @@ static int get_exec_path (char *exec_path, const size_t exec_path_sz)
 
   const size_t len = strlen (exec_path);
 
+  #elif defined (__OpenBSD__)
+
+  int mib[4];
+
+  mib[0] = CTL_KERN;
+  mib[1] = KERN_PROC_ARGS;
+  mib[2] = getpid ();
+  mib[3] = KERN_PROC_ARGV;
+
+  size_t size = 0;
+
+  if (sysctl (mib, 4, NULL, &size, NULL, 0) == -1) return -1;
+
+  char **argv = hcmalloc (size);
+
+  if (!argv) return -1;
+
+  if (sysctl (mib, 4, argv, &size, NULL, 0) == -1)
+  {
+    free (argv);
+
+    return -1;
+  }
+
+  if (argv[0])
+  {
+    strncpy (exec_path, argv[0], exec_path_sz - 1);
+
+    exec_path[exec_path_sz - 1] = '\0';
+  }
+
+  free (argv);
+
+  const size_t len = strlen (exec_path);
+
   #else
   #error Your Operating System is not supported or detected
   #endif
@@ -582,21 +621,27 @@ int hc_mkdir (const char *name, MAYBE_UNUSED const int mode)
 
 int hc_mkdir_rec (const char *path, MAYBE_UNUSED const int mode)
 {
+  if (hc_mkdir (path, mode) == 0) return 0;
+
+  if (errno == EEXIST) return 0;
+
   char *fullpath = hcstrdup (path);
 
   char *subpath = dirname (fullpath);
 
-  if (strlen (subpath) > 1)
+  if (hc_mkdir_rec (subpath, mode) == -1)
   {
-    if (hc_mkdir_rec (subpath, mode) == -1) return -1;
+    hcfree (fullpath);
+
+    return -1;
   }
 
+  hcfree (fullpath);
+
   if (hc_mkdir (path, mode) == -1)
   {
     if (errno != EEXIST) return -1;
   }
 
-  hcfree (fullpath);
-
   return 0;
 }
