@@ -241,6 +241,17 @@ int save_hash (hashcat_ctx_t *hashcat_ctx)
           hc_fputc (separator, &fp);
         }
 
+        if (user_options->dynamic_x == true)
+        {
+          dynamicx_t *dynamicx = hashes->hash_info[idx]->dynamicx;
+
+          u32 i;
+
+          for (i = 0; i < dynamicx->dynamicx_len; i++) hc_fputc (dynamicx->dynamicx_buf[i], &fp);
+
+          hc_fputc (separator, &fp);
+        }
+
         const int out_len = hash_encode (hashcat_ctx->hashconfig, hashcat_ctx->hashes, hashcat_ctx->module_ctx, (char *) out_buf, HCBUFSIZ_LARGE, salt_pos, digest_pos);
 
         out_buf[out_len] = 0;
@@ -323,7 +334,7 @@ int check_hash (hashcat_ctx_t *hashcat_ctx, hc_device_param_t *device_param, pla
 
     if (device_param->is_cuda == true)
     {
-      rc = hc_cuMemcpyDtoHAsync (hashcat_ctx, tmps, device_param->cuda_d_tmps + (plain->gidvid * hashconfig->tmp_size), hashconfig->tmp_size, device_param->cuda_stream);
+      rc = hc_cuMemcpyDtoH (hashcat_ctx, tmps, device_param->cuda_d_tmps + (plain->gidvid * hashconfig->tmp_size), hashconfig->tmp_size);
 
       if (rc == 0)
       {
@@ -340,7 +351,7 @@ int check_hash (hashcat_ctx_t *hashcat_ctx, hc_device_param_t *device_param, pla
 
     if (device_param->is_hip == true)
     {
-      rc = hc_hipMemcpyDtoHAsync (hashcat_ctx, tmps, device_param->hip_d_tmps + (plain->gidvid * hashconfig->tmp_size), hashconfig->tmp_size, device_param->hip_stream);
+      rc = hc_hipMemcpyDtoH (hashcat_ctx, tmps, device_param->hip_d_tmps + (plain->gidvid * hashconfig->tmp_size), hashconfig->tmp_size);
 
       if (rc == 0)
       {
@@ -355,9 +366,23 @@ int check_hash (hashcat_ctx_t *hashcat_ctx, hc_device_param_t *device_param, pla
       }
     }
 
+    #if defined (__APPLE__)
+    if (device_param->is_metal == true)
+    {
+      rc = hc_mtlMemcpyDtoH (hashcat_ctx, device_param->metal_device, device_param->metal_command_queue, tmps, device_param->metal_d_tmps, plain->gidvid * hashconfig->tmp_size, hashconfig->tmp_size);
+
+      if (rc == -1)
+      {
+        hcfree (tmps);
+
+        return -1;
+      }
+    }
+    #endif
+
     if (device_param->is_opencl == true)
     {
-      rc = hc_clEnqueueReadBuffer (hashcat_ctx, device_param->opencl_command_queue, device_param->opencl_d_tmps, CL_FALSE, plain->gidvid * hashconfig->tmp_size, hashconfig->tmp_size, tmps, 0, NULL, &opencl_event);
+      rc = hc_clEnqueueReadBuffer (hashcat_ctx, device_param->opencl_command_queue, device_param->opencl_d_tmps, CL_TRUE, plain->gidvid * hashconfig->tmp_size, hashconfig->tmp_size, tmps, 0, NULL, &opencl_event);
 
       if (rc == 0)
       {
@@ -562,18 +587,25 @@ int check_cracked (hashcat_ctx_t *hashcat_ctx, hc_device_param_t *device_param)
 
   if (device_param->is_cuda == true)
   {
-    if (hc_cuMemcpyDtoHAsync (hashcat_ctx, &num_cracked, device_param->cuda_d_result, sizeof (u32), device_param->cuda_stream) == -1) return -1;
+    if (hc_cuMemcpyDtoH (hashcat_ctx, &num_cracked, device_param->cuda_d_result, sizeof (u32)) == -1) return -1;
 
     if (hc_cuStreamSynchronize (hashcat_ctx, device_param->cuda_stream) == -1) return -1;
   }
 
   if (device_param->is_hip == true)
   {
-    if (hc_hipMemcpyDtoHAsync (hashcat_ctx, &num_cracked, device_param->hip_d_result, sizeof (u32), device_param->hip_stream) == -1) return -1;
+    if (hc_hipMemcpyDtoH (hashcat_ctx, &num_cracked, device_param->hip_d_result, sizeof (u32)) == -1) return -1;
 
     if (hc_hipStreamSynchronize (hashcat_ctx, device_param->hip_stream) == -1) return -1;
   }
 
+  #if defined (__APPLE__)
+  if (device_param->is_metal == true)
+  {
+    if (hc_mtlMemcpyDtoH (hashcat_ctx, device_param->metal_device, device_param->metal_command_queue, &num_cracked, device_param->metal_d_result, 0, sizeof (u32)) == -1) return -1;
+  }
+  #endif
+
   if (device_param->is_opencl == true)
   {
     /* blocking */
@@ -592,7 +624,7 @@ int check_cracked (hashcat_ctx_t *hashcat_ctx, hc_device_param_t *device_param)
 
   if (device_param->is_cuda == true)
   {
-    rc = hc_cuMemcpyDtoHAsync (hashcat_ctx, cracked, device_param->cuda_d_plain_bufs, num_cracked * sizeof (plain_t), device_param->cuda_stream);
+    rc = hc_cuMemcpyDtoH (hashcat_ctx, cracked, device_param->cuda_d_plain_bufs, num_cracked * sizeof (plain_t));
 
     if (rc == 0)
     {
@@ -609,7 +641,7 @@ int check_cracked (hashcat_ctx_t *hashcat_ctx, hc_device_param_t *device_param)
 
   if (device_param->is_hip == true)
   {
-    rc = hc_hipMemcpyDtoHAsync (hashcat_ctx, cracked, device_param->hip_d_plain_bufs, num_cracked * sizeof (plain_t), device_param->hip_stream);
+    rc = hc_hipMemcpyDtoH (hashcat_ctx, cracked, device_param->hip_d_plain_bufs, num_cracked * sizeof (plain_t));
 
     if (rc == 0)
     {
@@ -624,6 +656,20 @@ int check_cracked (hashcat_ctx_t *hashcat_ctx, hc_device_param_t *device_param)
     }
   }
 
+  #if defined (__APPLE__)
+  if (device_param->is_metal == true)
+  {
+    rc = hc_mtlMemcpyDtoH (hashcat_ctx, device_param->metal_device, device_param->metal_command_queue, cracked, device_param->metal_d_plain_bufs, 0, num_cracked * sizeof (plain_t));
+
+    if (rc == -1)
+    {
+      hcfree (cracked);
+
+      return -1;
+    }
+  }
+  #endif
+
   if (device_param->is_opencl == true)
   {
     /* blocking */
@@ -656,6 +702,8 @@ int check_cracked (hashcat_ctx_t *hashcat_ctx, hc_device_param_t *device_param)
 
       hashes->digests_done++;
 
+      hashes->digests_done_new++;
+
       cpt_cracked++;
 
       salt_buf->digests_done++;
@@ -703,6 +751,18 @@ int check_cracked (hashcat_ctx_t *hashcat_ctx, hc_device_param_t *device_param)
         }
       }
 
+      #if defined (__APPLE__)
+      if (device_param->is_metal == true)
+      {
+        rc = run_metal_kernel_memset32 (hashcat_ctx, device_param, device_param->metal_d_digests_shown, salt_buf->digests_offset * sizeof (u32), 0, salt_buf->digests_cnt * sizeof (u32));
+
+        if (rc == -1)
+        {
+          break;
+        }
+      }
+      #endif
+
       if (device_param->is_opencl == true)
       {
         /* NOTE: run_opencl_kernel_bzero() does not handle buffer offset */
@@ -751,6 +811,13 @@ int check_cracked (hashcat_ctx_t *hashcat_ctx, hc_device_param_t *device_param)
     if (run_hip_kernel_bzero (hashcat_ctx, device_param, device_param->hip_d_result, sizeof (u32)) == -1) return -1;
   }
 
+  #if defined (__APPLE__)
+  if (device_param->is_metal == true)
+  {
+    if (run_metal_kernel_bzero (hashcat_ctx, device_param, device_param->metal_d_result, sizeof (u32)) == -1) return -1;
+  }
+  #endif
+
   if (device_param->is_opencl == true)
   {
     if (run_opencl_kernel_bzero (hashcat_ctx, device_param, device_param->opencl_d_result, sizeof (u32)) == -1) return -1;
@@ -815,6 +882,8 @@ int hashes_init_filename (hashcat_ctx_t *hashcat_ctx)
     }
   }
 
+  hashes->parser_token_length_cnt = 0;
+
   return 0;
 }
 
@@ -945,7 +1014,7 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
   void   *esalts_buf     = NULL;
   void   *hook_salts_buf = NULL;
 
-  if ((user_options->username == true) || (hashconfig->opts_type & OPTS_TYPE_HASH_COPY) || (hashconfig->opts_type & OPTS_TYPE_HASH_SPLIT))
+  if ((user_options->dynamic_x == true) || (user_options->username == true) || (hashconfig->opts_type & OPTS_TYPE_HASH_COPY) || (hashconfig->opts_type & OPTS_TYPE_HASH_SPLIT))
   {
     u64 hash_pos;
 
@@ -955,6 +1024,11 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
 
       hashes_buf[hash_pos].hash_info = hash_info;
 
+      if (user_options->dynamic_x == true)
+      {
+        hash_info->dynamicx = (dynamicx_t *) hcmalloc (sizeof (dynamicx_t));
+      }
+
       if (user_options->username == true)
       {
         hash_info->user = (user_t *) hcmalloc (sizeof (user_t));
@@ -1059,7 +1133,7 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
 
     hashes_cnt = 1;
   }
-  else if (user_options->hash_info == true)
+  else if (user_options->hash_info > 0)
   {
   }
   else if (user_options->keyspace == true)
@@ -1068,7 +1142,7 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
   else if (user_options->stdout_flag == true)
   {
   }
-  else if (user_options->backend_info == true)
+  else if (user_options->backend_info > 0)
   {
   }
   else
@@ -1174,6 +1248,20 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
 
             if (parser_status == PARSER_OK)
             {
+              if (module_ctx->module_hash_decode_postprocess != MODULE_DEFAULT)
+              {
+                parser_status = module_ctx->module_hash_decode_postprocess (hashconfig, hash->digest, hash->salt, hash->esalt, hash->hook_salt, hash->hash_info, user_options, user_options_extra);
+
+                if (parser_status == PARSER_OK)
+                {
+                  // nothing to do
+                }
+                else
+                {
+                  event_log_warning (hashcat_ctx, "Hash '%s': %s", input_buf, strparser (parser_status));
+                }
+              }
+
               hashes_buf[hashes_cnt].hash_info->split->split_group  = 0;
               hashes_buf[hashes_cnt].hash_info->split->split_origin = SPLIT_ORIGIN_LEFT;
 
@@ -1184,12 +1272,31 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
               event_log_warning (hashcat_ctx, "Hash '%s': %s", input_buf, strparser (parser_status));
             }
 
+            if (parser_status == PARSER_TOKEN_LENGTH)
+            {
+              hashes->parser_token_length_cnt++;
+            }
+
             hash = &hashes_buf[hashes_cnt];
 
             parser_status = module_ctx->module_hash_decode (hashconfig, hash->digest, hash->salt, hash->esalt, hash->hook_salt, hash->hash_info, hash_buf + 16, 16);
 
             if (parser_status == PARSER_OK)
             {
+              if (module_ctx->module_hash_decode_postprocess != MODULE_DEFAULT)
+              {
+                parser_status = module_ctx->module_hash_decode_postprocess (hashconfig, hash->digest, hash->salt, hash->esalt, hash->hook_salt, hash->hash_info, user_options, user_options_extra);
+
+                if (parser_status == PARSER_OK)
+                {
+                  // nothing to do
+                }
+                else
+                {
+                  event_log_warning (hashcat_ctx, "Hash '%s': %s", input_buf, strparser (parser_status));
+                }
+              }
+
               hashes_buf[hashes_cnt].hash_info->split->split_group  = 0;
               hashes_buf[hashes_cnt].hash_info->split->split_origin = SPLIT_ORIGIN_RIGHT;
 
@@ -1199,6 +1306,11 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
             {
               event_log_warning (hashcat_ctx, "Hash '%s': %s", input_buf, strparser (parser_status));
             }
+
+            if (parser_status == PARSER_TOKEN_LENGTH)
+            {
+              hashes->parser_token_length_cnt++;
+            }
           }
           else
           {
@@ -1208,6 +1320,20 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
 
             if (parser_status == PARSER_OK)
             {
+              if (module_ctx->module_hash_decode_postprocess != MODULE_DEFAULT)
+              {
+                parser_status = module_ctx->module_hash_decode_postprocess (hashconfig, hash->digest, hash->salt, hash->esalt, hash->hook_salt, hash->hash_info, user_options, user_options_extra);
+
+                if (parser_status == PARSER_OK)
+                {
+                  // nothing to do
+                }
+                else
+                {
+                  event_log_warning (hashcat_ctx, "Hash '%s': %s", input_buf, strparser (parser_status));
+                }
+              }
+
               hashes_buf[hashes_cnt].hash_info->split->split_group  = 0;
               hashes_buf[hashes_cnt].hash_info->split->split_origin = SPLIT_ORIGIN_NONE;
 
@@ -1217,6 +1343,11 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
             {
               event_log_warning (hashcat_ctx, "Hash '%s': %s", input_buf, strparser (parser_status));
             }
+
+            if (parser_status == PARSER_TOKEN_LENGTH)
+            {
+              hashes->parser_token_length_cnt++;
+            }
           }
         }
         else
@@ -1227,12 +1358,31 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
 
           if (parser_status == PARSER_OK)
           {
+            if (module_ctx->module_hash_decode_postprocess != MODULE_DEFAULT)
+            {
+              parser_status = module_ctx->module_hash_decode_postprocess (hashconfig, hash->digest, hash->salt, hash->esalt, hash->hook_salt, hash->hash_info, user_options, user_options_extra);
+
+              if (parser_status == PARSER_OK)
+              {
+                // nothing to do
+              }
+              else
+              {
+                event_log_warning (hashcat_ctx, "Hash '%s': %s", input_buf, strparser (parser_status));
+              }
+            }
+
             hashes_cnt++;
           }
           else
           {
             event_log_warning (hashcat_ctx, "Hash '%s': %s", input_buf, strparser (parser_status));
           }
+
+          if (parser_status == PARSER_TOKEN_LENGTH)
+          {
+            hashes->parser_token_length_cnt++;
+          }
         }
       }
     }
@@ -1387,6 +1537,33 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
               continue;
             }
 
+            if (module_ctx->module_hash_decode_postprocess != MODULE_DEFAULT)
+            {
+              int parser_status_postprocess = module_ctx->module_hash_decode_postprocess (hashconfig, hash->digest, hash->salt, hash->esalt, hash->hook_salt, hash->hash_info, user_options, user_options_extra);
+
+              if (parser_status_postprocess < PARSER_GLOBAL_ZERO)
+              {
+                char *tmp_line_buf;
+
+                hc_asprintf (&tmp_line_buf, "%s", line_buf);
+
+                compress_terminal_line_length (tmp_line_buf, 38, 32);
+
+                if (user_options->machine_readable == true)
+                {
+                  event_log_warning (hashcat_ctx, "%s:%u:%s:%s", hashes->hashfile, line_num, tmp_line_buf, strparser (parser_status_postprocess));
+                }
+                else
+                {
+                  event_log_warning (hashcat_ctx, "Hashfile '%s' on line %u (%s): %s", hashes->hashfile, line_num, tmp_line_buf, strparser (parser_status_postprocess));
+                }
+
+                hcfree (tmp_line_buf);
+
+                continue;
+              }
+            }
+
             hashes_buf[hashes_cnt].hash_info->split->split_group  = line_num;
             hashes_buf[hashes_cnt].hash_info->split->split_origin = SPLIT_ORIGIN_LEFT;
 
@@ -1418,6 +1595,33 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
               continue;
             }
 
+            if (module_ctx->module_hash_decode_postprocess != MODULE_DEFAULT)
+            {
+              int parser_status_postprocess = module_ctx->module_hash_decode_postprocess (hashconfig, hash->digest, hash->salt, hash->esalt, hash->hook_salt, hash->hash_info, user_options, user_options_extra);
+
+              if (parser_status_postprocess < PARSER_GLOBAL_ZERO)
+              {
+                char *tmp_line_buf;
+
+                hc_asprintf (&tmp_line_buf, "%s", line_buf);
+
+                compress_terminal_line_length (tmp_line_buf, 38, 32);
+
+                if (user_options->machine_readable == true)
+                {
+                  event_log_warning (hashcat_ctx, "%s:%u:%s:%s", hashes->hashfile, line_num, tmp_line_buf, strparser (parser_status_postprocess));
+                }
+                else
+                {
+                  event_log_warning (hashcat_ctx, "Hashfile '%s' on line %u (%s): %s", hashes->hashfile, line_num, tmp_line_buf, strparser (parser_status_postprocess));
+                }
+
+                hcfree (tmp_line_buf);
+
+                continue;
+              }
+            }
+
             hashes_buf[hashes_cnt].hash_info->split->split_group  = line_num;
             hashes_buf[hashes_cnt].hash_info->split->split_origin = SPLIT_ORIGIN_RIGHT;
 
@@ -1451,6 +1655,33 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
               continue;
             }
 
+            if (module_ctx->module_hash_decode_postprocess != MODULE_DEFAULT)
+            {
+              int parser_status_postprocess = module_ctx->module_hash_decode_postprocess (hashconfig, hash->digest, hash->salt, hash->esalt, hash->hook_salt, hash->hash_info, user_options, user_options_extra);
+
+              if (parser_status_postprocess < PARSER_GLOBAL_ZERO)
+              {
+                char *tmp_line_buf;
+
+                hc_asprintf (&tmp_line_buf, "%s", line_buf);
+
+                compress_terminal_line_length (tmp_line_buf, 38, 32);
+
+                if (user_options->machine_readable == true)
+                {
+                  event_log_warning (hashcat_ctx, "%s:%u:%s:%s", hashes->hashfile, line_num, tmp_line_buf, strparser (parser_status_postprocess));
+                }
+                else
+                {
+                  event_log_warning (hashcat_ctx, "Hashfile '%s' on line %u (%s): %s", hashes->hashfile, line_num, tmp_line_buf, strparser (parser_status_postprocess));
+                }
+
+                hcfree (tmp_line_buf);
+
+                continue;
+              }
+            }
+
             hashes_buf[hashes_cnt].hash_info->split->split_group  = line_num;
             hashes_buf[hashes_cnt].hash_info->split->split_origin = SPLIT_ORIGIN_NONE;
 
@@ -1482,9 +1713,46 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
 
             hcfree (tmp_line_buf);
 
+            if (parser_status == PARSER_TOKEN_LENGTH)
+            {
+              hashes->parser_token_length_cnt++;
+            }
+
             continue;
           }
 
+          if (module_ctx->module_hash_decode_postprocess != MODULE_DEFAULT)
+          {
+            int parser_status_postprocess = module_ctx->module_hash_decode_postprocess (hashconfig, hash->digest, hash->salt, hash->esalt, hash->hook_salt, hash->hash_info, user_options, user_options_extra);
+
+            if (parser_status_postprocess < PARSER_GLOBAL_ZERO)
+            {
+              char *tmp_line_buf;
+
+              hc_asprintf (&tmp_line_buf, "%s", line_buf);
+
+              compress_terminal_line_length (tmp_line_buf, 38, 32);
+
+              if (user_options->machine_readable == true)
+              {
+                event_log_warning (hashcat_ctx, "%s:%u:%s:%s", hashes->hashfile, line_num, tmp_line_buf, strparser (parser_status_postprocess));
+              }
+              else
+              {
+                event_log_warning (hashcat_ctx, "Hashfile '%s' on line %u (%s): %s", hashes->hashfile, line_num, tmp_line_buf, strparser (parser_status_postprocess));
+              }
+
+              hcfree (tmp_line_buf);
+
+              if (parser_status_postprocess == PARSER_TOKEN_LENGTH)
+              {
+                hashes->parser_token_length_cnt++;
+              }
+
+              continue;
+            }
+          }
+
           hashes_cnt++;
         }
 
@@ -1570,12 +1838,31 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
 
         if (parser_status == PARSER_OK)
         {
+          if (module_ctx->module_hash_decode_postprocess != MODULE_DEFAULT)
+          {
+            parser_status = module_ctx->module_hash_decode_postprocess (hashconfig, hash->digest, hash->salt, hash->esalt, hash->hook_salt, hash->hash_info, user_options, user_options_extra);
+
+            if (parser_status == PARSER_OK)
+            {
+              // nothing to do
+            }
+            else
+            {
+              event_log_warning (hashcat_ctx, "Hash '%s': %s", input_buf, strparser (parser_status));
+            }
+          }
+
           hashes_cnt++;
         }
         else
         {
           event_log_warning (hashcat_ctx, "Hash '%s': %s", input_buf, strparser (parser_status));
         }
+
+        if (parser_status == PARSER_TOKEN_LENGTH)
+        {
+          hashes->parser_token_length_cnt++;
+        }
       }
     }
   }
@@ -1625,6 +1912,16 @@ int hashes_init_stage1 (hashcat_ctx_t *hashcat_ctx)
     }
   }
 
+  if (hashes->parser_token_length_cnt > 0)
+  {
+    event_log_advice (hashcat_ctx, NULL); // we can guarantee that the previous line was not an empty line
+    event_log_advice (hashcat_ctx, "* Token length exception: %u/%u hashes", hashes->parser_token_length_cnt, hashes->parser_token_length_cnt + hashes->hashes_cnt);
+    event_log_advice (hashcat_ctx, "  This error happens if the wrong hash type is specified, if the hashes are");
+    event_log_advice (hashcat_ctx, "  malformed, or if input is otherwise not as expected (for example, if the");
+    event_log_advice (hashcat_ctx, "  --username or --dynamic-x option is used but no username or dynamic-tag is present)");
+    event_log_advice (hashcat_ctx, NULL);
+  }
+
   return 0;
 }
 
@@ -1716,14 +2013,14 @@ int hashes_init_stage2 (hashcat_ctx_t *hashcat_ctx)
   u32 digests_cnt  = hashes_cnt;
   u32 digests_done = 0;
 
-  u32 *digests_shown     = (u32 *) hccalloc (digests_cnt, sizeof (u32));
+  u32 *digests_shown = (u32 *) hccalloc (digests_cnt, sizeof (u32));
 
   u32 salts_cnt   = 0;
   u32 salts_done  = 0;
 
   hashinfo_t **hash_info = NULL;
 
-  if ((user_options->username == true) || (hashconfig->opts_type & OPTS_TYPE_HASH_COPY) || (hashconfig->opts_type & OPTS_TYPE_HASH_SPLIT))
+  if ((user_options->username == true) || (user_options->dynamic_x == true) || (hashconfig->opts_type & OPTS_TYPE_HASH_COPY) || (hashconfig->opts_type & OPTS_TYPE_HASH_SPLIT))
   {
     hash_info = (hashinfo_t **) hccalloc (hashes_cnt, sizeof (hashinfo_t *));
   }
@@ -1871,16 +2168,17 @@ int hashes_init_stage3 (hashcat_ctx_t *hashcat_ctx)
 {
   hashes_t *hashes = hashcat_ctx->hashes;
 
-  u32  digests_done  = hashes->digests_done;
-  u32 *digests_shown = hashes->digests_shown;
+  u32  digests_done      = hashes->digests_done;
+  u32  digests_done_zero = hashes->digests_done_zero;
+  u32  digests_done_pot  = hashes->digests_done_pot;
+  u32 *digests_shown     = hashes->digests_shown;
 
-  u32  salts_cnt     = hashes->salts_cnt;
-  u32  salts_done    = hashes->salts_done;
-  u32 *salts_shown   = hashes->salts_shown;
+  u32  salts_cnt         = hashes->salts_cnt;
+  u32  salts_done        = hashes->salts_done;
+  u32 *salts_shown       = hashes->salts_shown;
 
-  hash_t *hashes_buf = hashes->hashes_buf;
-
-  salt_t *salts_buf  = hashes->salts_buf;
+  hash_t *hashes_buf     = hashes->hashes_buf;
+  salt_t *salts_buf      = hashes->salts_buf;
 
   for (u32 salt_idx = 0; salt_idx < salts_cnt; salt_idx++)
   {
@@ -1892,12 +2190,25 @@ int hashes_init_stage3 (hashcat_ctx_t *hashcat_ctx)
     {
       const u32 hashes_idx = salt_buf->digests_offset + digest_idx;
 
-      if (hashes_buf[hashes_idx].cracked == 1)
+      if (hashes_buf[hashes_idx].cracked_pot == 1)
       {
         digests_shown[hashes_idx] = 1;
 
         digests_done++;
 
+        digests_done_pot++;
+
+        salt_buf->digests_done++;
+      }
+
+      if (hashes_buf[hashes_idx].cracked_zero == 1)
+      {
+        digests_shown[hashes_idx] = 1;
+
+        digests_done++;
+
+        digests_done_zero++;
+
         salt_buf->digests_done++;
       }
     }
@@ -1912,10 +2223,11 @@ int hashes_init_stage3 (hashcat_ctx_t *hashcat_ctx)
     if (salts_done == salts_cnt) mycracked (hashcat_ctx);
   }
 
-  hashes->digests_done = digests_done;
+  hashes->digests_done      = digests_done;
+  hashes->digests_done_zero = digests_done_zero;
+  hashes->digests_done_pot  = digests_done_pot;
 
-  hashes->salts_cnt   = salts_cnt;
-  hashes->salts_done  = salts_done;
+  hashes->salts_done        = salts_done;
 
   return 0;
 }
@@ -1924,9 +2236,7 @@ int hashes_init_stage4 (hashcat_ctx_t *hashcat_ctx)
 {
   hashconfig_t         *hashconfig         = hashcat_ctx->hashconfig;
   hashes_t             *hashes             = hashcat_ctx->hashes;
-  module_ctx_t         *module_ctx         = hashcat_ctx->module_ctx;
   user_options_t       *user_options       = hashcat_ctx->user_options;
-  user_options_extra_t *user_options_extra = hashcat_ctx->user_options_extra;
 
   if (hashes->salts_cnt == 1)
     hashconfig->opti_type |= OPTI_TYPE_SINGLE_SALT;
@@ -1967,6 +2277,27 @@ int hashes_init_stage4 (hashcat_ctx_t *hashcat_ctx)
     }
   }
 
+  // https://github.com/hashcat/hashcat/issues/3641
+
+  if ((hashconfig->opts_type & OPTS_TYPE_DEEP_COMP_KERNEL) == 0)
+  {
+    if ((hashconfig->opts_type & OPTS_TYPE_MULTIHASH_DESPITE_ESALT) == 0)
+    {
+      if (hashconfig->attack_exec == ATTACK_EXEC_OUTSIDE_KERNEL)
+      {
+        if (hashconfig->esalt_size > 0)
+        {
+          if (hashes->digests_cnt != hashes->salts_cnt)
+          {
+            event_log_error (hashcat_ctx, "This hash-mode plugin cannot crack multiple hashes with the same salt, please select one of the hashes.");
+
+            return -1;
+          }
+        }
+      }
+    }
+  }
+
   // test iteration count in association attack
 
   if (user_options->attack_mode == ATTACK_MODE_ASSOCIATION)
@@ -1984,22 +2315,6 @@ int hashes_init_stage4 (hashcat_ctx_t *hashcat_ctx)
     }
   }
 
-  // time to update extra_tmp_size which is tmp_size value based on hash configuration
-
-  if (module_ctx->module_extra_tmp_size != MODULE_DEFAULT)
-  {
-    const u64 extra_tmp_size = module_ctx->module_extra_tmp_size (hashconfig, user_options, user_options_extra, hashes);
-
-    if (extra_tmp_size == (u64) -1)
-    {
-      event_log_error (hashcat_ctx, "Mixed hash settings are not supported.");
-
-      return -1;
-    }
-
-    hashconfig->tmp_size = extra_tmp_size;
-  }
-
   // at this point we no longer need hash_t* structure
 
   hash_t *hashes_buf = hashes->hashes_buf;
@@ -2035,6 +2350,96 @@ int hashes_init_stage4 (hashcat_ctx_t *hashcat_ctx)
   return 0;
 }
 
+int hashes_init_stage5 (hashcat_ctx_t *hashcat_ctx)
+{
+  hashconfig_t         *hashconfig         = hashcat_ctx->hashconfig;
+  hashes_t             *hashes             = hashcat_ctx->hashes;
+  module_ctx_t         *module_ctx         = hashcat_ctx->module_ctx;
+  user_options_t       *user_options       = hashcat_ctx->user_options;
+  user_options_extra_t *user_options_extra = hashcat_ctx->user_options_extra;
+
+  // time to update extra_tmp_size which is tmp_size value based on hash configuration
+
+  if (module_ctx->module_extra_tmp_size != MODULE_DEFAULT)
+  {
+    const u64 extra_tmp_size = module_ctx->module_extra_tmp_size (hashconfig, user_options, user_options_extra, hashes);
+
+    if ((extra_tmp_size & (1ULL << 62)) || (extra_tmp_size & (1ULL << 63)))
+    {
+      const u64 salt_pos = extra_tmp_size & 0xffffffff;
+
+      char *tmp_buf = (char *) hcmalloc (HCBUFSIZ_LARGE);
+
+      const int tmp_len = hash_encode (hashcat_ctx->hashconfig, hashcat_ctx->hashes, hashcat_ctx->module_ctx, tmp_buf, HCBUFSIZ_LARGE, salt_pos, 0);
+
+      tmp_buf[tmp_len] = 0;
+
+      compress_terminal_line_length (tmp_buf, 47, 6);
+
+      char *user_hash = strdup (tmp_buf);
+
+      if (extra_tmp_size & (1ULL << 62))
+      {
+        strncpy (tmp_buf, hashconfig->st_hash, HCBUFSIZ_LARGE - 1);
+
+        compress_terminal_line_length (tmp_buf, 47, 6);
+
+        char *st_hash = strdup (tmp_buf);
+
+        event_log_error (hashcat_ctx, "ERROR: Incompatible self-test configuration detected.");
+
+        event_log_warning (hashcat_ctx, "The specified target hash:");
+        event_log_warning (hashcat_ctx, "  -> %s", user_hash);
+        event_log_warning (hashcat_ctx, "does not match the configuration of the self-test hash:");
+        event_log_warning (hashcat_ctx, "  -> %s", st_hash);
+        event_log_warning (hashcat_ctx, "The JIT-compiled kernel for this configuration may be incompatible.");
+        event_log_warning (hashcat_ctx, "You must disable the self-test functionality or recompile the plugin with a matching self-test hash.");
+        event_log_warning (hashcat_ctx, "To disable the self-test, use the --self-test-disable option.");
+        event_log_warning (hashcat_ctx, NULL);
+
+        hcfree (tmp_buf);
+        hcfree (user_hash);
+        hcfree (st_hash);
+
+        return -1;
+      }
+
+      if (extra_tmp_size & (1ULL << 63))
+      {
+        const int tmp_len = hash_encode (hashcat_ctx->hashconfig, hashcat_ctx->hashes, hashcat_ctx->module_ctx, tmp_buf, HCBUFSIZ_LARGE, 0, 0);
+
+        tmp_buf[tmp_len] = 0;
+
+        compress_terminal_line_length (tmp_buf, 47, 6);
+
+        char *user_hash2 = strdup (tmp_buf);
+
+        event_log_error (hashcat_ctx, "ERROR: Mixed configuration detected.");
+
+        event_log_warning (hashcat_ctx, "The specified target hash:");
+        event_log_warning (hashcat_ctx, "  -> %s", user_hash);
+        event_log_warning (hashcat_ctx, "does not match the configuration of another target hash:");
+        event_log_warning (hashcat_ctx, "  -> %s", user_hash2);
+        event_log_warning (hashcat_ctx, "Please run these hashes in separate cracking sessions.");
+        event_log_warning (hashcat_ctx, NULL);
+
+        hcfree (tmp_buf);
+        hcfree (user_hash);
+        hcfree (user_hash2);
+
+        return -1;
+      }
+
+      hcfree (tmp_buf);
+      hcfree (user_hash);
+    }
+
+    hashconfig->tmp_size = extra_tmp_size;
+  }
+
+  return 0;
+}
+
 int hashes_init_selftest (hashcat_ctx_t *hashcat_ctx)
 {
   folder_config_t *folder_config = hashcat_ctx->folder_config;
@@ -2233,7 +2638,7 @@ int hashes_init_zerohash (hashcat_ctx_t *hashcat_ctx)
   hash_t *hashes_buf = hashes->hashes_buf;
   u32     hashes_cnt = hashes->hashes_cnt;
 
-  // no solution for these special hash types (for instane because they use hashfile in output etc)
+  // no solution for these special hash types (for instance because they use hashfile in output etc)
 
   hash_t hash_buf;
 
@@ -2274,7 +2679,7 @@ int hashes_init_zerohash (hashcat_ctx_t *hashcat_ctx)
       next->pw_buf = (char *) hcmalloc (1);
       next->pw_len = 0;
 
-      next->cracked = 1;
+      next->cracked_zero = 1;
 
       // should we show the cracked zero hash to the user?
 
@@ -2395,6 +2800,8 @@ void hashes_logger (hashcat_ctx_t *hashcat_ctx)
   logfile_top_uint   (hashes->hashlist_format);
   logfile_top_uint   (hashes->hashes_cnt);
   logfile_top_uint   (hashes->digests_cnt);
+  logfile_top_uint   (hashes->digests_done_pot);
+  logfile_top_uint   (hashes->digests_done_zero);
   logfile_top_uint   (hashes->digests_done);
   logfile_top_uint   (hashes->salts_cnt);
   logfile_top_uint   (hashes->salts_done);
