@@ -20,7 +20,9 @@ static const u32   HASH_CATEGORY  = HASH_CATEGORY_PASSWORD_MANAGER;
 static const char *HASH_NAME      = "Password Safe v2";
 static const u64   KERN_TYPE      = 9000;
 static const u32   OPTI_TYPE      = OPTI_TYPE_ZERO_BYTE;
-static const u64   OPTS_TYPE      = OPTS_TYPE_PT_GENERATE_LE
+static const u64   OPTS_TYPE      = OPTS_TYPE_STOCK_MODULE
+                                  | OPTS_TYPE_PT_GENERATE_LE
+                                  | OPTS_TYPE_NATIVE_THREADS
                                   | OPTS_TYPE_BINARY_HASHFILE
                                   | OPTS_TYPE_AUTODETECT_DISABLE
                                   | OPTS_TYPE_DYNAMIC_SHARED;
@@ -65,6 +67,8 @@ typedef struct pwsafe2_tmp
 
 } pwsafe2_tmp_t;
 
+#include "blowfish_common.c"
+
 u64 module_tmp_size (MAYBE_UNUSED const hashconfig_t *hashconfig, MAYBE_UNUSED const user_options_t *user_options, MAYBE_UNUSED const user_options_extra_t *user_options_extra)
 {
   const u64 tmp_size = (const u64) sizeof (pwsafe2_tmp_t);
@@ -72,99 +76,6 @@ u64 module_tmp_size (MAYBE_UNUSED const hashconfig_t *hashconfig, MAYBE_UNUSED c
   return tmp_size;
 }
 
-char *module_jit_build_options (MAYBE_UNUSED const hashconfig_t *hashconfig, MAYBE_UNUSED const user_options_t *user_options, MAYBE_UNUSED const user_options_extra_t *user_options_extra, MAYBE_UNUSED const hashes_t *hashes, MAYBE_UNUSED const hc_device_param_t *device_param)
-{
-  char *jit_build_options = NULL;
-
-  // this mode heavily depends on the available shared memory size
-  // note the kernel need to have some special code changes in order to make use to use post-48k memory region
-  // we need to set some macros
-
-  bool use_dynamic = false;
-
-  if (device_param->is_cuda == true)
-  {
-    use_dynamic = true;
-  }
-
-  // this uses some nice feedback effect.
-  // based on the device_local_mem_size the reqd_work_group_size in the kernel is set to some value
-  // which is then is read from the opencl host in the kernel_preferred_wgs_multiple1/2/3 result.
-  // therefore we do not need to set module_kernel_threads_min/max except for CPU, where the threads are set to fixed 1.
-
-  if (device_param->opencl_device_type & CL_DEVICE_TYPE_CPU)
-  {
-    hc_asprintf (&jit_build_options, "-D FIXED_LOCAL_SIZE=%u", 1);
-  }
-  else
-  {
-    u32 overhead = 0;
-
-    if (device_param->opencl_device_vendor_id == VENDOR_ID_NV)
-    {
-      // note we need to use device_param->device_local_mem_size - 4 because opencl jit returns with:
-      // Entry function '...' uses too much shared data (0xc004 bytes, 0xc000 max)
-      // on my development system. no clue where the 4 bytes are spent.
-      // I did some research on this and it seems to be related with the datatype.
-      // For example, if i used u8 instead, there's only 1 byte wasted.
-
-      if (device_param->is_opencl == true)
-      {
-        overhead = 1;
-      }
-    }
-
-    if (user_options->kernel_threads_chgd == true)
-    {
-      u32 fixed_local_size = user_options->kernel_threads;
-
-      if (use_dynamic == true)
-      {
-        if ((fixed_local_size * 4096) > device_param->kernel_dynamic_local_mem_size_memset)
-        {
-          // otherwise out-of-bound reads
-
-          fixed_local_size = device_param->kernel_dynamic_local_mem_size_memset / 4096;
-        }
-
-        hc_asprintf (&jit_build_options, "-D FIXED_LOCAL_SIZE=%u -D DYNAMIC_LOCAL", fixed_local_size);
-      }
-      else
-      {
-        if ((fixed_local_size * 4096) > (device_param->device_local_mem_size - overhead))
-        {
-          // otherwise out-of-bound reads
-
-          fixed_local_size = (device_param->device_local_mem_size - overhead) / 4096;
-        }
-
-        hc_asprintf (&jit_build_options, "-D FIXED_LOCAL_SIZE=%u", fixed_local_size);
-      }
-    }
-    else
-    {
-      if (use_dynamic == true)
-      {
-        // using kernel_dynamic_local_mem_size_memset is a bit hackish.
-        // we had to brute-force this value out of an already loaded CUDA function.
-        // there's no official way to query for this value.
-
-        const u32 fixed_local_size = device_param->kernel_dynamic_local_mem_size_memset / 4096;
-
-        hc_asprintf (&jit_build_options, "-D FIXED_LOCAL_SIZE=%u -D DYNAMIC_LOCAL", fixed_local_size);
-      }
-      else
-      {
-        const u32 fixed_local_size = (device_param->device_local_mem_size - overhead) / 4096;
-
-        hc_asprintf (&jit_build_options, "-D FIXED_LOCAL_SIZE=%u", fixed_local_size);
-      }
-    }
-  }
-
-  return jit_build_options;
-}
-
 bool module_outfile_check_disable (MAYBE_UNUSED const hashconfig_t *hashconfig, MAYBE_UNUSED const user_options_t *user_options, MAYBE_UNUSED const user_options_extra_t *user_options_extra)
 {
   const bool outfile_check_disable = true;
@@ -187,7 +98,7 @@ int module_hash_decode (MAYBE_UNUSED const hashconfig_t *hashconfig, MAYBE_UNUSE
 
   HCFILE fp;
 
-  if (hc_fopen (&fp, (const char *) line_buf, "rb") == false) return (PARSER_HASH_FILE);
+  if (hc_fopen (&fp, line_buf, "rb") == false) return (PARSER_HASH_FILE);
 
   psafe2_hdr buf;
 
@@ -223,7 +134,10 @@ void module_init (module_ctx_t *module_ctx)
   module_ctx->module_benchmark_esalt          = MODULE_DEFAULT;
   module_ctx->module_benchmark_hook_salt      = MODULE_DEFAULT;
   module_ctx->module_benchmark_mask           = MODULE_DEFAULT;
+  module_ctx->module_benchmark_charset        = MODULE_DEFAULT;
   module_ctx->module_benchmark_salt           = MODULE_DEFAULT;
+  module_ctx->module_bridge_name              = MODULE_DEFAULT;
+  module_ctx->module_bridge_type              = MODULE_DEFAULT;
   module_ctx->module_build_plain_postprocess  = MODULE_DEFAULT;
   module_ctx->module_deep_comp_kernel         = MODULE_DEFAULT;
   module_ctx->module_deprecated_notice        = MODULE_DEFAULT;
@@ -241,6 +155,7 @@ void module_init (module_ctx_t *module_ctx)
   module_ctx->module_hash_binary_count        = MODULE_DEFAULT;
   module_ctx->module_hash_binary_parse        = MODULE_DEFAULT;
   module_ctx->module_hash_binary_save         = MODULE_DEFAULT;
+  module_ctx->module_hash_decode_postprocess  = MODULE_DEFAULT;
   module_ctx->module_hash_decode_potfile      = MODULE_DEFAULT;
   module_ctx->module_hash_decode_zero_hash    = MODULE_DEFAULT;
   module_ctx->module_hash_decode              = module_hash_decode;
@@ -261,7 +176,7 @@ void module_init (module_ctx_t *module_ctx)
   module_ctx->module_hook23                   = MODULE_DEFAULT;
   module_ctx->module_hook_salt_size           = MODULE_DEFAULT;
   module_ctx->module_hook_size                = MODULE_DEFAULT;
-  module_ctx->module_jit_build_options        = module_jit_build_options;
+  module_ctx->module_jit_build_options        = blowfish_module_jit_build_options;
   module_ctx->module_jit_cache_disable        = MODULE_DEFAULT;
   module_ctx->module_kernel_accel_max         = MODULE_DEFAULT;
   module_ctx->module_kernel_accel_min         = MODULE_DEFAULT;
