@@ -21,7 +21,8 @@ static const char *HASH_NAME      = "iTunes backup < 10.0";
 static const u64   KERN_TYPE      = 14700;
 static const u32   OPTI_TYPE      = OPTI_TYPE_ZERO_BYTE
                                   | OPTI_TYPE_SLOW_HASH_SIMD_LOOP;
-static const u64   OPTS_TYPE      = OPTS_TYPE_PT_GENERATE_LE
+static const u64   OPTS_TYPE      = OPTS_TYPE_STOCK_MODULE
+                                  | OPTS_TYPE_PT_GENERATE_LE
                                   | OPTS_TYPE_ST_HEX;
 static const u32   SALT_TYPE      = SALT_TYPE_EMBEDDED;
 static const char *ST_PASS        = "hashcat";
@@ -61,17 +62,6 @@ typedef struct pbkdf2_sha1_tmp
 
 static const char *SIGNATURE_ITUNES_BACKUP = "$itunes_backup$";
 
-bool module_unstable_warning (MAYBE_UNUSED const hashconfig_t *hashconfig, MAYBE_UNUSED const user_options_t *user_options, MAYBE_UNUSED const user_options_extra_t *user_options_extra, MAYBE_UNUSED const hc_device_param_t *device_param)
-{
-  // AMD Radeon Pro W5700X Compute Engine; 1.2 (Apr 22 2021 21:54:44); 11.3.1; 20E241
-  if ((device_param->opencl_platform_vendor_id == VENDOR_ID_APPLE) && (device_param->opencl_device_type & CL_DEVICE_TYPE_GPU))
-  {
-    return true;
-  }
-
-  return false;
-}
-
 u64 module_esalt_size (MAYBE_UNUSED const hashconfig_t *hashconfig, MAYBE_UNUSED const user_options_t *user_options, MAYBE_UNUSED const user_options_extra_t *user_options_extra)
 {
   const u64 esalt_size = (const u64) sizeof (itunes_backup_t);
@@ -86,68 +76,57 @@ u64 module_tmp_size (MAYBE_UNUSED const hashconfig_t *hashconfig, MAYBE_UNUSED c
   return tmp_size;
 }
 
-u32 module_pw_max (MAYBE_UNUSED const hashconfig_t *hashconfig, MAYBE_UNUSED const user_options_t *user_options, MAYBE_UNUSED const user_options_extra_t *user_options_extra)
-{
-  // this overrides the reductions of PW_MAX in case optimized kernel is selected
-  // IOW, even in optimized kernel mode it support length 256
-
-  const u32 pw_max = PW_MAX;
-
-  return pw_max;
-}
-
 int module_hash_decode (MAYBE_UNUSED const hashconfig_t *hashconfig, MAYBE_UNUSED void *digest_buf, MAYBE_UNUSED salt_t *salt, MAYBE_UNUSED void *esalt_buf, MAYBE_UNUSED void *hook_salt_buf, MAYBE_UNUSED hashinfo_t *hash_info, const char *line_buf, MAYBE_UNUSED const int line_len)
 {
   u32 *digest = (u32 *) digest_buf;
 
   itunes_backup_t *itunes_backup = (itunes_backup_t *) esalt_buf;
 
-  token_t token;
+  hc_token_t token;
+
+  memset (&token, 0, sizeof (hc_token_t));
 
   token.token_cnt = 7;
 
   token.signatures_cnt    = 1;
   token.signatures_buf[0] = SIGNATURE_ITUNES_BACKUP;
 
-  token.len_min[0] = 15;
-  token.len_max[0] = 15;
   token.sep[0]     = '*';
-  token.attr[0]    = TOKEN_ATTR_VERIFY_LENGTH
+  token.len[0]     = 15;
+  token.attr[0]    = TOKEN_ATTR_FIXED_LENGTH
                    | TOKEN_ATTR_VERIFY_SIGNATURE;
 
+  token.sep[1]     = '*';
   token.len_min[1] = 1;
   token.len_max[1] = 2;
-  token.sep[1]     = '*';
   token.attr[1]    = TOKEN_ATTR_VERIFY_LENGTH
                    | TOKEN_ATTR_VERIFY_DIGIT;
 
-  token.len_min[2] = 80;
-  token.len_max[2] = 80;
   token.sep[2]     = '*';
-  token.attr[2]    = TOKEN_ATTR_VERIFY_LENGTH
+  token.len[2]     = 80;
+  token.attr[2]    = TOKEN_ATTR_FIXED_LENGTH
                    | TOKEN_ATTR_VERIFY_HEX;
 
+  token.sep[3]     = '*';
   token.len_min[3] = 1;
   token.len_max[3] = 6;
-  token.sep[3]     = '*';
   token.attr[3]    = TOKEN_ATTR_VERIFY_LENGTH
                    | TOKEN_ATTR_VERIFY_DIGIT;
 
-  token.len_min[4] = 40;
-  token.len_max[4] = 40;
   token.sep[4]     = '*';
-  token.attr[4]    = TOKEN_ATTR_VERIFY_LENGTH
+  token.len[4]     = 40;
+  token.attr[4]    = TOKEN_ATTR_FIXED_LENGTH
                    | TOKEN_ATTR_VERIFY_HEX;
 
+  token.sep[5]     = '*';
   token.len_min[5] = 0;
   token.len_max[5] = 10;
-  token.sep[5]     = '*';
   token.attr[5]    = TOKEN_ATTR_VERIFY_LENGTH
                    | TOKEN_ATTR_VERIFY_DIGIT;
 
+  token.sep[6]     = '*';
   token.len_min[6] = 0;
   token.len_max[6] = 40;
-  token.sep[6]     = '*';
   token.attr[6]    = TOKEN_ATTR_VERIFY_LENGTH
                    | TOKEN_ATTR_VERIFY_HEX;
 
@@ -180,16 +159,16 @@ int module_hash_decode (MAYBE_UNUSED const hashconfig_t *hashconfig, MAYBE_UNUSE
 
   u32 *wpky_buf_ptr = (u32 *) itunes_backup->wpky;
 
-  wpky_buf_ptr[0] = hex_to_u32 ((const u8 *) &wpky_pos[ 0]);
-  wpky_buf_ptr[1] = hex_to_u32 ((const u8 *) &wpky_pos[ 8]);
-  wpky_buf_ptr[2] = hex_to_u32 ((const u8 *) &wpky_pos[16]);
-  wpky_buf_ptr[3] = hex_to_u32 ((const u8 *) &wpky_pos[24]);
-  wpky_buf_ptr[4] = hex_to_u32 ((const u8 *) &wpky_pos[32]);
-  wpky_buf_ptr[5] = hex_to_u32 ((const u8 *) &wpky_pos[40]);
-  wpky_buf_ptr[6] = hex_to_u32 ((const u8 *) &wpky_pos[48]);
-  wpky_buf_ptr[7] = hex_to_u32 ((const u8 *) &wpky_pos[56]);
-  wpky_buf_ptr[8] = hex_to_u32 ((const u8 *) &wpky_pos[64]);
-  wpky_buf_ptr[9] = hex_to_u32 ((const u8 *) &wpky_pos[72]);
+  wpky_buf_ptr[0] = hex_to_u32 (&wpky_pos[ 0]);
+  wpky_buf_ptr[1] = hex_to_u32 (&wpky_pos[ 8]);
+  wpky_buf_ptr[2] = hex_to_u32 (&wpky_pos[16]);
+  wpky_buf_ptr[3] = hex_to_u32 (&wpky_pos[24]);
+  wpky_buf_ptr[4] = hex_to_u32 (&wpky_pos[32]);
+  wpky_buf_ptr[5] = hex_to_u32 (&wpky_pos[40]);
+  wpky_buf_ptr[6] = hex_to_u32 (&wpky_pos[48]);
+  wpky_buf_ptr[7] = hex_to_u32 (&wpky_pos[56]);
+  wpky_buf_ptr[8] = hex_to_u32 (&wpky_pos[64]);
+  wpky_buf_ptr[9] = hex_to_u32 (&wpky_pos[72]);
 
   wpky_buf_ptr[0] = byte_swap_32 (wpky_buf_ptr[0]);
   wpky_buf_ptr[1] = byte_swap_32 (wpky_buf_ptr[1]);
@@ -265,11 +244,11 @@ int module_hash_decode (MAYBE_UNUSED const hashconfig_t *hashconfig, MAYBE_UNUSE
 
     u32 *dpsl_buf_ptr = (u32 *) itunes_backup->dpsl;
 
-    dpsl_buf_ptr[0] = hex_to_u32 ((const u8 *) &dpsl_pos[ 0]);
-    dpsl_buf_ptr[1] = hex_to_u32 ((const u8 *) &dpsl_pos[ 8]);
-    dpsl_buf_ptr[2] = hex_to_u32 ((const u8 *) &dpsl_pos[16]);
-    dpsl_buf_ptr[3] = hex_to_u32 ((const u8 *) &dpsl_pos[24]);
-    dpsl_buf_ptr[4] = hex_to_u32 ((const u8 *) &dpsl_pos[32]);
+    dpsl_buf_ptr[0] = hex_to_u32 (&dpsl_pos[ 0]);
+    dpsl_buf_ptr[1] = hex_to_u32 (&dpsl_pos[ 8]);
+    dpsl_buf_ptr[2] = hex_to_u32 (&dpsl_pos[16]);
+    dpsl_buf_ptr[3] = hex_to_u32 (&dpsl_pos[24]);
+    dpsl_buf_ptr[4] = hex_to_u32 (&dpsl_pos[32]);
 
     dpsl_buf_ptr[0] = byte_swap_32 (dpsl_buf_ptr[ 0]);
     dpsl_buf_ptr[1] = byte_swap_32 (dpsl_buf_ptr[ 1]);
@@ -354,7 +333,10 @@ void module_init (module_ctx_t *module_ctx)
   module_ctx->module_benchmark_esalt          = MODULE_DEFAULT;
   module_ctx->module_benchmark_hook_salt      = MODULE_DEFAULT;
   module_ctx->module_benchmark_mask           = MODULE_DEFAULT;
+  module_ctx->module_benchmark_charset        = MODULE_DEFAULT;
   module_ctx->module_benchmark_salt           = MODULE_DEFAULT;
+  module_ctx->module_bridge_name              = MODULE_DEFAULT;
+  module_ctx->module_bridge_type              = MODULE_DEFAULT;
   module_ctx->module_build_plain_postprocess  = MODULE_DEFAULT;
   module_ctx->module_deep_comp_kernel         = MODULE_DEFAULT;
   module_ctx->module_deprecated_notice        = MODULE_DEFAULT;
@@ -372,6 +354,7 @@ void module_init (module_ctx_t *module_ctx)
   module_ctx->module_hash_binary_count        = MODULE_DEFAULT;
   module_ctx->module_hash_binary_parse        = MODULE_DEFAULT;
   module_ctx->module_hash_binary_save         = MODULE_DEFAULT;
+  module_ctx->module_hash_decode_postprocess  = MODULE_DEFAULT;
   module_ctx->module_hash_decode_potfile      = MODULE_DEFAULT;
   module_ctx->module_hash_decode_zero_hash    = MODULE_DEFAULT;
   module_ctx->module_hash_decode              = module_hash_decode;
@@ -410,7 +393,7 @@ void module_init (module_ctx_t *module_ctx)
   module_ctx->module_potfile_disable          = MODULE_DEFAULT;
   module_ctx->module_potfile_keep_all_hashes  = MODULE_DEFAULT;
   module_ctx->module_pwdump_column            = MODULE_DEFAULT;
-  module_ctx->module_pw_max                   = module_pw_max;
+  module_ctx->module_pw_max                   = MODULE_DEFAULT;
   module_ctx->module_pw_min                   = MODULE_DEFAULT;
   module_ctx->module_salt_max                 = MODULE_DEFAULT;
   module_ctx->module_salt_min                 = MODULE_DEFAULT;
@@ -419,6 +402,6 @@ void module_init (module_ctx_t *module_ctx)
   module_ctx->module_st_hash                  = module_st_hash;
   module_ctx->module_st_pass                  = module_st_pass;
   module_ctx->module_tmp_size                 = module_tmp_size;
-  module_ctx->module_unstable_warning         = module_unstable_warning;
+  module_ctx->module_unstable_warning         = MODULE_DEFAULT;
   module_ctx->module_warmup_disable           = MODULE_DEFAULT;
 }
