@@ -10,6 +10,7 @@
 #include "bitops.h"
 #include "convert.h"
 #include "shared.h"
+#include "memory.h"
 
 static const u32   ATTACK_EXEC    = ATTACK_EXEC_OUTSIDE_KERNEL;
 static const u32   DGST_POS0      = 0;
@@ -24,8 +25,8 @@ static const u32   OPTI_TYPE      = OPTI_TYPE_ZERO_BYTE;
 static const u64   OPTS_TYPE      = OPTS_TYPE_STOCK_MODULE
                                   | OPTS_TYPE_PT_GENERATE_LE
                                   | OPTS_TYPE_MP_MULTI_DISABLE
+                                  | OPTS_TYPE_NATIVE_THREADS
                                   | OPTS_TYPE_LOOP_PREPARE
-                                  | OPTS_TYPE_SELF_TEST_DISABLE
                                   | OPTS_TYPE_ST_HEX;
 static const u32   SALT_TYPE      = SALT_TYPE_EMBEDDED;
 static const char *ST_PASS        = "hashcat";
@@ -55,41 +56,9 @@ typedef struct ethereum_scrypt
 
 static const char *SIGNATURE_ETHEREUM_SCRYPT = "$ethereum$s";
 
-static const u64 SCRYPT_N = 262144;
-static const u64 SCRYPT_R = 8;
-static const u64 SCRYPT_P = 1;
+static const u32 SCRYPT_THREADS = 4;
 
-bool module_unstable_warning (MAYBE_UNUSED const hashconfig_t *hashconfig, MAYBE_UNUSED const user_options_t *user_options, MAYBE_UNUSED const user_options_extra_t *user_options_extra, MAYBE_UNUSED const hc_device_param_t *device_param)
-{
-  // AMD Radeon Pro W5700X Compute Engine; 1.2 (Apr 22 2021 21:54:44); 11.3.1; 20E241
-  if ((device_param->opencl_platform_vendor_id == VENDOR_ID_APPLE) && (device_param->opencl_device_type & CL_DEVICE_TYPE_GPU))
-  {
-    return true;
-  }
-
-  return false;
-}
-
-u32 module_kernel_loops_min (MAYBE_UNUSED const hashconfig_t *hashconfig, MAYBE_UNUSED const user_options_t *user_options, MAYBE_UNUSED const user_options_extra_t *user_options_extra)
-{
-  const u32 kernel_loops_min = 1024;
-
-  return kernel_loops_min;
-}
-
-u32 module_kernel_loops_max (MAYBE_UNUSED const hashconfig_t *hashconfig, MAYBE_UNUSED const user_options_t *user_options, MAYBE_UNUSED const user_options_extra_t *user_options_extra)
-{
-  const u32 kernel_loops_max = 1024;
-
-  return kernel_loops_max;
-}
-
-u32 module_kernel_threads_max (MAYBE_UNUSED const hashconfig_t *hashconfig, MAYBE_UNUSED const user_options_t *user_options, MAYBE_UNUSED const user_options_extra_t *user_options_extra)
-{
-  const u32 kernel_threads_max = 4;
-
-  return kernel_threads_max;
-}
+#include "scrypt_common.c"
 
 u64 module_esalt_size (MAYBE_UNUSED const hashconfig_t *hashconfig, MAYBE_UNUSED const user_options_t *user_options, MAYBE_UNUSED const user_options_extra_t *user_options_extra)
 {
@@ -98,198 +67,6 @@ u64 module_esalt_size (MAYBE_UNUSED const hashconfig_t *hashconfig, MAYBE_UNUSED
   return esalt_size;
 }
 
-u32 module_pw_max (MAYBE_UNUSED const hashconfig_t *hashconfig, MAYBE_UNUSED const user_options_t *user_options, MAYBE_UNUSED const user_options_extra_t *user_options_extra)
-{
-  // this overrides the reductions of PW_MAX in case optimized kernel is selected
-  // IOW, even in optimized kernel mode it support length 256
-
-  const u32 pw_max = PW_MAX;
-
-  return pw_max;
-}
-
-u64 module_extra_buffer_size (MAYBE_UNUSED const hashconfig_t *hashconfig, MAYBE_UNUSED const user_options_t *user_options, MAYBE_UNUSED const user_options_extra_t *user_options_extra, MAYBE_UNUSED const hashes_t *hashes, MAYBE_UNUSED const hc_device_param_t *device_param)
-{
-  // we need to set the self-test hash settings to pass the self-test
-  // the decoder for the self-test is called after this function
-
-  const u64 scrypt_N = (hashes->salts_buf[0].scrypt_N) ? hashes->salts_buf[0].scrypt_N : SCRYPT_N;
-  const u64 scrypt_r = (hashes->salts_buf[0].scrypt_r) ? hashes->salts_buf[0].scrypt_r : SCRYPT_R;
-
-  const u64 kernel_power_max = ((OPTS_TYPE & OPTS_TYPE_MP_MULTI_DISABLE) ? 1 : device_param->device_processors) * device_param->kernel_threads_max * device_param->kernel_accel_max;
-
-  u64 tmto_start = 0;
-  u64 tmto_stop  = 4;
-
-  if (user_options->scrypt_tmto_chgd == true)
-  {
-    tmto_start = user_options->scrypt_tmto;
-    tmto_stop  = user_options->scrypt_tmto;
-  }
-
-  // size_pws
-
-  const u64 size_pws = kernel_power_max * sizeof (pw_t);
-
-  const u64 size_pws_amp = size_pws;
-
-  // size_pws_comp
-
-  const u64 size_pws_comp = kernel_power_max * (sizeof (u32) * 64);
-
-  // size_pws_idx
-
-  const u64 size_pws_idx = (kernel_power_max + 1) * sizeof (pw_idx_t);
-
-  // size_tmps
-
-  const u64 size_tmps = kernel_power_max * hashconfig->tmp_size;
-
-  // size_hooks
-
-  const u64 size_hooks = kernel_power_max * hashconfig->hook_size;
-
-  u64 size_pws_pre  = 4;
-  u64 size_pws_base = 4;
-
-  if (user_options->slow_candidates == true)
-  {
-    // size_pws_pre
-
-    size_pws_pre = kernel_power_max * sizeof (pw_pre_t);
-
-    // size_pws_base
-
-    size_pws_base = kernel_power_max * sizeof (pw_pre_t);
-  }
-
-  // sometimes device_available_mem and device_maxmem_alloc reported back from the opencl runtime are a bit inaccurate.
-  // let's add some extra space just to be sure.
-  // now depends on the kernel-accel value (where scrypt and similar benefits), but also hard minimum 64mb and maximum 1024mb limit
-
-  u64 EXTRA_SPACE = (1024ULL * 1024ULL) * device_param->kernel_accel_max;
-
-  EXTRA_SPACE = MAX (EXTRA_SPACE, (  64ULL * 1024ULL * 1024ULL));
-  EXTRA_SPACE = MIN (EXTRA_SPACE, (1024ULL * 1024ULL * 1024ULL));
-
-  const u64 scrypt_extra_space
-    = device_param->size_bfs
-    + device_param->size_combs
-    + device_param->size_digests
-    + device_param->size_esalts
-    + device_param->size_markov_css
-    + device_param->size_plains
-    + device_param->size_results
-    + device_param->size_root_css
-    + device_param->size_rules
-    + device_param->size_rules_c
-    + device_param->size_salts
-    + device_param->size_shown
-    + device_param->size_tm
-    + device_param->size_st_digests
-    + device_param->size_st_salts
-    + device_param->size_st_esalts
-    + size_pws
-    + size_pws_amp
-    + size_pws_comp
-    + size_pws_idx
-    + size_tmps
-    + size_hooks
-    + size_pws_pre
-    + size_pws_base
-    + EXTRA_SPACE;
-
-  bool not_enough_memory = true;
-
-  u64 size_scrypt = 0;
-
-  u64 tmto;
-
-  for (tmto = tmto_start; tmto <= tmto_stop; tmto++)
-  {
-    size_scrypt = (128ULL * scrypt_r) * scrypt_N;
-
-    size_scrypt /= 1ull << tmto;
-
-    size_scrypt *= kernel_power_max;
-
-    if ((size_scrypt / 4) > device_param->device_maxmem_alloc) continue;
-
-    if ((size_scrypt + scrypt_extra_space) > device_param->device_available_mem) continue;
-
-    not_enough_memory = false;
-
-    break;
-  }
-
-  if (not_enough_memory == true) return -1;
-
-  return size_scrypt;
-}
-
-u64 module_tmp_size (MAYBE_UNUSED const hashconfig_t *hashconfig, MAYBE_UNUSED const user_options_t *user_options, MAYBE_UNUSED const user_options_extra_t *user_options_extra)
-{
-  const u64 tmp_size = 0; // we'll add some later
-
-  return tmp_size;
-}
-
-u64 module_extra_tmp_size (MAYBE_UNUSED const hashconfig_t *hashconfig, MAYBE_UNUSED const user_options_t *user_options, MAYBE_UNUSED const user_options_extra_t *user_options_extra, MAYBE_UNUSED const hashes_t *hashes)
-{
-  const u64 scrypt_N = (hashes->salts_buf[0].scrypt_N) ? hashes->salts_buf[0].scrypt_N : SCRYPT_N;
-  const u64 scrypt_r = (hashes->salts_buf[0].scrypt_r) ? hashes->salts_buf[0].scrypt_r : SCRYPT_R;
-  const u64 scrypt_p = (hashes->salts_buf[0].scrypt_p) ? hashes->salts_buf[0].scrypt_p : SCRYPT_P;
-
-  // we need to check that all hashes have the same scrypt settings
-
-  for (u32 i = 1; i < hashes->salts_cnt; i++)
-  {
-    if ((hashes->salts_buf[i].scrypt_N != scrypt_N)
-     || (hashes->salts_buf[i].scrypt_r != scrypt_r)
-     || (hashes->salts_buf[i].scrypt_p != scrypt_p))
-    {
-      return -1;
-    }
-  }
-
-  const u64 tmp_size = 128ULL * scrypt_r * scrypt_p;
-
-  return tmp_size;
-}
-
-bool module_warmup_disable (MAYBE_UNUSED const hashconfig_t *hashconfig, MAYBE_UNUSED const user_options_t *user_options, MAYBE_UNUSED const user_options_extra_t *user_options_extra)
-{
-  return true;
-}
-
-char *module_jit_build_options (MAYBE_UNUSED const hashconfig_t *hashconfig, MAYBE_UNUSED const user_options_t *user_options, MAYBE_UNUSED const user_options_extra_t *user_options_extra, MAYBE_UNUSED const hashes_t *hashes, MAYBE_UNUSED const hc_device_param_t *device_param)
-{
-  const u64 scrypt_N = (hashes->salts_buf[0].scrypt_N) ? hashes->salts_buf[0].scrypt_N : SCRYPT_N;
-  const u64 scrypt_r = (hashes->salts_buf[0].scrypt_r) ? hashes->salts_buf[0].scrypt_r : SCRYPT_R;
-  const u64 scrypt_p = (hashes->salts_buf[0].scrypt_p) ? hashes->salts_buf[0].scrypt_p : SCRYPT_P;
-
-  const u64 extra_buffer_size = device_param->extra_buffer_size;
-
-  const u64 kernel_power_max = ((OPTS_TYPE & OPTS_TYPE_MP_MULTI_DISABLE) ? 1 : device_param->device_processors) * device_param->kernel_threads_max * device_param->kernel_accel_max;
-
-  const u64 size_scrypt = 128ULL * scrypt_r * scrypt_N;
-
-  const u64 scrypt_tmto_final = (kernel_power_max * size_scrypt) / extra_buffer_size;
-
-  const u64 tmp_size = 128ULL * scrypt_r * scrypt_p;
-
-  char *jit_build_options = NULL;
-
-  hc_asprintf (&jit_build_options, "-D SCRYPT_N=%u -D SCRYPT_R=%u -D SCRYPT_P=%u -D SCRYPT_TMTO=%" PRIu64 " -D SCRYPT_TMP_ELEM=%" PRIu64,
-    hashes->salts_buf[0].scrypt_N,
-    hashes->salts_buf[0].scrypt_r,
-    hashes->salts_buf[0].scrypt_p,
-    scrypt_tmto_final,
-    tmp_size / 16);
-
-  return jit_build_options;
-}
-
 int module_hash_decode (MAYBE_UNUSED const hashconfig_t *hashconfig, MAYBE_UNUSED void *digest_buf, MAYBE_UNUSED salt_t *salt, MAYBE_UNUSED void *esalt_buf, MAYBE_UNUSED void *hook_salt_buf, MAYBE_UNUSED hashinfo_t *hash_info, const char *line_buf, MAYBE_UNUSED const int line_len)
 {
   u32 *digest = (u32 *) digest_buf;
@@ -298,15 +75,16 @@ int module_hash_decode (MAYBE_UNUSED const hashconfig_t *hashconfig, MAYBE_UNUSE
 
   hc_token_t token;
 
+  memset (&token, 0, sizeof (hc_token_t));
+
   token.token_cnt  = 7;
 
   token.signatures_cnt    = 1;
   token.signatures_buf[0] = SIGNATURE_ETHEREUM_SCRYPT;
 
   token.sep[0]     = '*';
-  token.len_min[0] = 11;
-  token.len_max[0] = 11;
-  token.attr[0]    = TOKEN_ATTR_VERIFY_LENGTH
+  token.len[0]     = 11;
+  token.attr[0]    = TOKEN_ATTR_FIXED_LENGTH
                    | TOKEN_ATTR_VERIFY_SIGNATURE;
 
   token.sep[1]     = '*';
@@ -328,21 +106,18 @@ int module_hash_decode (MAYBE_UNUSED const hashconfig_t *hashconfig, MAYBE_UNUSE
                    | TOKEN_ATTR_VERIFY_DIGIT;
 
   token.sep[4]     = '*';
-  token.len_min[4] = 64;
-  token.len_max[4] = 64;
-  token.attr[4]    = TOKEN_ATTR_VERIFY_LENGTH
+  token.len[4]     = 64;
+  token.attr[4]    = TOKEN_ATTR_FIXED_LENGTH
                    | TOKEN_ATTR_VERIFY_HEX;
 
   token.sep[5]     = '*';
-  token.len_min[5] = 64;
-  token.len_max[5] = 64;
-  token.attr[5]    = TOKEN_ATTR_VERIFY_LENGTH
+  token.len[5]     = 64;
+  token.attr[5]    = TOKEN_ATTR_FIXED_LENGTH
                    | TOKEN_ATTR_VERIFY_HEX;
 
   token.sep[6]     = '*';
-  token.len_min[6] = 64;
-  token.len_max[6] = 64;
-  token.attr[6]    = TOKEN_ATTR_VERIFY_LENGTH
+  token.len[6]     = 64;
+  token.attr[6]    = TOKEN_ATTR_FIXED_LENGTH
                    | TOKEN_ATTR_VERIFY_HEX;
 
   const int rc_tokenizer = input_tokenizer ((const u8 *) line_buf, line_len, &token);
@@ -390,27 +165,27 @@ int module_hash_decode (MAYBE_UNUSED const hashconfig_t *hashconfig, MAYBE_UNUSE
 
   const u8 *ciphertext_pos = token.buf[5];
 
-  ethereum_scrypt->ciphertext[0] = hex_to_u32 ((const u8 *) &ciphertext_pos[ 0]);
-  ethereum_scrypt->ciphertext[1] = hex_to_u32 ((const u8 *) &ciphertext_pos[ 8]);
-  ethereum_scrypt->ciphertext[2] = hex_to_u32 ((const u8 *) &ciphertext_pos[16]);
-  ethereum_scrypt->ciphertext[3] = hex_to_u32 ((const u8 *) &ciphertext_pos[24]);
-  ethereum_scrypt->ciphertext[4] = hex_to_u32 ((const u8 *) &ciphertext_pos[32]);
-  ethereum_scrypt->ciphertext[5] = hex_to_u32 ((const u8 *) &ciphertext_pos[40]);
-  ethereum_scrypt->ciphertext[6] = hex_to_u32 ((const u8 *) &ciphertext_pos[48]);
-  ethereum_scrypt->ciphertext[7] = hex_to_u32 ((const u8 *) &ciphertext_pos[56]);
+  ethereum_scrypt->ciphertext[0] = hex_to_u32 (&ciphertext_pos[ 0]);
+  ethereum_scrypt->ciphertext[1] = hex_to_u32 (&ciphertext_pos[ 8]);
+  ethereum_scrypt->ciphertext[2] = hex_to_u32 (&ciphertext_pos[16]);
+  ethereum_scrypt->ciphertext[3] = hex_to_u32 (&ciphertext_pos[24]);
+  ethereum_scrypt->ciphertext[4] = hex_to_u32 (&ciphertext_pos[32]);
+  ethereum_scrypt->ciphertext[5] = hex_to_u32 (&ciphertext_pos[40]);
+  ethereum_scrypt->ciphertext[6] = hex_to_u32 (&ciphertext_pos[48]);
+  ethereum_scrypt->ciphertext[7] = hex_to_u32 (&ciphertext_pos[56]);
 
   // hash
 
   const u8 *hash_pos = token.buf[6];
 
-  digest[0] = hex_to_u32 ((const u8 *) &hash_pos[ 0]);
-  digest[1] = hex_to_u32 ((const u8 *) &hash_pos[ 8]);
-  digest[2] = hex_to_u32 ((const u8 *) &hash_pos[16]);
-  digest[3] = hex_to_u32 ((const u8 *) &hash_pos[24]);
-  digest[4] = hex_to_u32 ((const u8 *) &hash_pos[32]);
-  digest[5] = hex_to_u32 ((const u8 *) &hash_pos[40]);
-  digest[6] = hex_to_u32 ((const u8 *) &hash_pos[48]);
-  digest[7] = hex_to_u32 ((const u8 *) &hash_pos[56]);
+  digest[0] = hex_to_u32 (&hash_pos[ 0]);
+  digest[1] = hex_to_u32 (&hash_pos[ 8]);
+  digest[2] = hex_to_u32 (&hash_pos[16]);
+  digest[3] = hex_to_u32 (&hash_pos[24]);
+  digest[4] = hex_to_u32 (&hash_pos[32]);
+  digest[5] = hex_to_u32 (&hash_pos[40]);
+  digest[6] = hex_to_u32 (&hash_pos[48]);
+  digest[7] = hex_to_u32 (&hash_pos[56]);
 
   return (PARSER_OK);
 }
@@ -425,7 +200,7 @@ int module_hash_encode (MAYBE_UNUSED const hashconfig_t *hashconfig, MAYBE_UNUSE
 
   tmp_salt[salt_len] = 0;
 
-  ethereum_scrypt_t *ethereum_scrypt = (ethereum_scrypt_t *) esalt_buf;
+  const ethereum_scrypt_t *ethereum_scrypt = (const ethereum_scrypt_t *) esalt_buf;
 
   const int line_len = snprintf (line_buf, line_size, "%s*%u*%u*%u*%s*%08x%08x%08x%08x%08x%08x%08x%08x*%08x%08x%08x%08x%08x%08x%08x%08x",
     SIGNATURE_ETHEREUM_SCRYPT,
@@ -465,6 +240,8 @@ void module_init (module_ctx_t *module_ctx)
   module_ctx->module_benchmark_mask           = MODULE_DEFAULT;
   module_ctx->module_benchmark_charset        = MODULE_DEFAULT;
   module_ctx->module_benchmark_salt           = MODULE_DEFAULT;
+  module_ctx->module_bridge_name              = MODULE_DEFAULT;
+  module_ctx->module_bridge_type              = MODULE_DEFAULT;
   module_ctx->module_build_plain_postprocess  = MODULE_DEFAULT;
   module_ctx->module_deep_comp_kernel         = MODULE_DEFAULT;
   module_ctx->module_deprecated_notice        = MODULE_DEFAULT;
@@ -475,9 +252,9 @@ void module_init (module_ctx_t *module_ctx)
   module_ctx->module_dgst_size                = module_dgst_size;
   module_ctx->module_dictstat_disable         = MODULE_DEFAULT;
   module_ctx->module_esalt_size               = module_esalt_size;
-  module_ctx->module_extra_buffer_size        = module_extra_buffer_size;
-  module_ctx->module_extra_tmp_size           = module_extra_tmp_size;
-  module_ctx->module_extra_tuningdb_block     = MODULE_DEFAULT;
+  module_ctx->module_extra_buffer_size        = scrypt_module_extra_buffer_size;
+  module_ctx->module_extra_tmp_size           = scrypt_module_extra_tmp_size;
+  module_ctx->module_extra_tuningdb_block     = scrypt_module_extra_tuningdb_block;
   module_ctx->module_forced_outfile_format    = MODULE_DEFAULT;
   module_ctx->module_hash_binary_count        = MODULE_DEFAULT;
   module_ctx->module_hash_binary_parse        = MODULE_DEFAULT;
@@ -503,13 +280,13 @@ void module_init (module_ctx_t *module_ctx)
   module_ctx->module_hook23                   = MODULE_DEFAULT;
   module_ctx->module_hook_salt_size           = MODULE_DEFAULT;
   module_ctx->module_hook_size                = MODULE_DEFAULT;
-  module_ctx->module_jit_build_options        = module_jit_build_options;
+  module_ctx->module_jit_build_options        = scrypt_module_jit_build_options;
   module_ctx->module_jit_cache_disable        = MODULE_DEFAULT;
   module_ctx->module_kernel_accel_max         = MODULE_DEFAULT;
   module_ctx->module_kernel_accel_min         = MODULE_DEFAULT;
-  module_ctx->module_kernel_loops_max         = module_kernel_loops_max;
-  module_ctx->module_kernel_loops_min         = module_kernel_loops_min;
-  module_ctx->module_kernel_threads_max       = module_kernel_threads_max;
+  module_ctx->module_kernel_loops_max         = scrypt_module_kernel_loops_max;
+  module_ctx->module_kernel_loops_min         = scrypt_module_kernel_loops_min;
+  module_ctx->module_kernel_threads_max       = scrypt_module_kernel_threads_max;
   module_ctx->module_kernel_threads_min       = MODULE_DEFAULT;
   module_ctx->module_kern_type                = module_kern_type;
   module_ctx->module_kern_type_dynamic        = MODULE_DEFAULT;
@@ -521,7 +298,7 @@ void module_init (module_ctx_t *module_ctx)
   module_ctx->module_potfile_disable          = MODULE_DEFAULT;
   module_ctx->module_potfile_keep_all_hashes  = MODULE_DEFAULT;
   module_ctx->module_pwdump_column            = MODULE_DEFAULT;
-  module_ctx->module_pw_max                   = module_pw_max;
+  module_ctx->module_pw_max                   = MODULE_DEFAULT;
   module_ctx->module_pw_min                   = MODULE_DEFAULT;
   module_ctx->module_salt_max                 = MODULE_DEFAULT;
   module_ctx->module_salt_min                 = MODULE_DEFAULT;
@@ -529,7 +306,7 @@ void module_init (module_ctx_t *module_ctx)
   module_ctx->module_separator                = MODULE_DEFAULT;
   module_ctx->module_st_hash                  = module_st_hash;
   module_ctx->module_st_pass                  = module_st_pass;
-  module_ctx->module_tmp_size                 = module_tmp_size;
-  module_ctx->module_unstable_warning         = module_unstable_warning;
-  module_ctx->module_warmup_disable           = module_warmup_disable;
+  module_ctx->module_tmp_size                 = scrypt_module_tmp_size;
+  module_ctx->module_unstable_warning         = MODULE_DEFAULT;
+  module_ctx->module_warmup_disable           = MODULE_DEFAULT;
 }
