@@ -28,7 +28,8 @@ static const u32   OPTI_TYPE      = OPTI_TYPE_ZERO_BYTE
                                   | OPTI_TYPE_NOT_ITERATED
                                   | OPTI_TYPE_NOT_SALTED
                                   | OPTI_TYPE_RAW_HASH;
-static const u64   OPTS_TYPE      = OPTS_TYPE_PT_GENERATE_LE
+static const u64   OPTS_TYPE      = OPTS_TYPE_STOCK_MODULE
+                                  | OPTS_TYPE_PT_GENERATE_LE
                                   | OPTS_TYPE_PT_ADD80
                                   | OPTS_TYPE_PT_ADDBITS14
                                   | OPTS_TYPE_AUTODETECT_DISABLE;
@@ -62,22 +63,36 @@ int module_hash_decode (MAYBE_UNUSED const hashconfig_t *hashconfig, MAYBE_UNUSE
 {
   u32 *digest = (u32 *) digest_buf;
 
-  token_t token;
+  hc_token_t token;
+
+  memset (&token, 0, sizeof (hc_token_t));
 
   token.token_cnt  = 1;
 
   token.len_min[0] = 1;
-  token.len_max[0] = 55;
+  token.len_max[0] = 55 * 2 + 6; /* 55 without $HEX[...] */
   token.attr[0]    = TOKEN_ATTR_VERIFY_LENGTH;
 
   const int rc_tokenizer = input_tokenizer ((const u8 *) line_buf, line_len, &token);
 
   if (rc_tokenizer != PARSER_OK) return (rc_tokenizer);
 
+  const u8 *unhex_buf = token.buf[0];
+  int unhex_len       = token.len[0];
+
+  if (is_hexify (unhex_buf, unhex_len))
+  {
+    unhex_len = exec_unhexify (unhex_buf, unhex_len, (u8 *) unhex_buf, unhex_len);
+  }
+  else if (unhex_len > 55)
+  {
+    return (PARSER_HASH_LENGTH);
+  }
+
   memset (digest, 0, hashconfig->dgst_size);
 
-  const u8 *pw_buf = token.buf[0];
-  const int pw_len = token.len[0];
+  const u8 *pw_buf = unhex_buf;
+  const int pw_len = unhex_len;
 
   memcpy ((char *) digest + 64, pw_buf, pw_len);
 
@@ -113,9 +128,35 @@ int module_hash_decode (MAYBE_UNUSED const hashconfig_t *hashconfig, MAYBE_UNUSE
 
 int module_hash_encode (MAYBE_UNUSED const hashconfig_t *hashconfig, MAYBE_UNUSED const void *digest_buf, MAYBE_UNUSED const salt_t *salt, MAYBE_UNUSED const void *esalt_buf, MAYBE_UNUSED const void *hook_salt_buf, MAYBE_UNUSED const hashinfo_t *hash_info, char *line_buf, MAYBE_UNUSED const int line_size)
 {
-  char *ptr = (char *) digest_buf;
+  const char *ptr = (const char *) digest_buf;
 
-  return snprintf (line_buf, line_size, "%s", ptr + 64);
+  const char *line_ptr = ptr + 64;
+  size_t line_len      = strnlen (line_ptr, 55);
+
+  if (need_hexify ((const u8 *) line_ptr, line_len, ':', 0))
+  {
+    char tmp_buf[55 * 2 + 6 + 1] = { 0 };
+
+    int tmp_len = 0;
+
+    tmp_buf[tmp_len++] = '$';
+    tmp_buf[tmp_len++] = 'H';
+    tmp_buf[tmp_len++] = 'E';
+    tmp_buf[tmp_len++] = 'X';
+    tmp_buf[tmp_len++] = '[';
+
+    exec_hexify ((const u8 *) line_ptr, line_len, (u8 *) tmp_buf + tmp_len);
+
+    tmp_len += line_len * 2;
+
+    tmp_buf[tmp_len++] = ']';
+
+    tmp_buf[tmp_len++] = 0;
+
+    return snprintf (line_buf, tmp_len, "%s", tmp_buf);
+  }
+
+  return snprintf (line_buf, line_size, "%s", line_ptr);
 }
 
 void module_init (module_ctx_t *module_ctx)
@@ -127,7 +168,10 @@ void module_init (module_ctx_t *module_ctx)
   module_ctx->module_benchmark_esalt          = MODULE_DEFAULT;
   module_ctx->module_benchmark_hook_salt      = MODULE_DEFAULT;
   module_ctx->module_benchmark_mask           = MODULE_DEFAULT;
+  module_ctx->module_benchmark_charset        = MODULE_DEFAULT;
   module_ctx->module_benchmark_salt           = MODULE_DEFAULT;
+  module_ctx->module_bridge_name              = MODULE_DEFAULT;
+  module_ctx->module_bridge_type              = MODULE_DEFAULT;
   module_ctx->module_build_plain_postprocess  = MODULE_DEFAULT;
   module_ctx->module_deep_comp_kernel         = MODULE_DEFAULT;
   module_ctx->module_deprecated_notice        = MODULE_DEFAULT;
@@ -145,6 +189,7 @@ void module_init (module_ctx_t *module_ctx)
   module_ctx->module_hash_binary_count        = MODULE_DEFAULT;
   module_ctx->module_hash_binary_parse        = MODULE_DEFAULT;
   module_ctx->module_hash_binary_save         = MODULE_DEFAULT;
+  module_ctx->module_hash_decode_postprocess  = MODULE_DEFAULT;
   module_ctx->module_hash_category            = module_hash_category;
   module_ctx->module_hash_decode              = module_hash_decode;
   module_ctx->module_hash_decode_potfile      = MODULE_DEFAULT;
