@@ -58,7 +58,7 @@ static int sort_by_hash_t_salt (const void *v1, const void *v2)
 }
 */
 
-// this function is special and only used whenever --username and --show are used together:
+// this function is special and only used whenever --username or --dynamic-x and --show are used together:
 // it will sort all tree entries according to the settings stored in hashconfig
 
 int sort_pot_tree_by_hash (const void *v1, const void *v2)
@@ -81,7 +81,10 @@ int sort_pot_orig_line (const void *v1, const void *v2)
   const pot_orig_line_entry_t *t1 = (const pot_orig_line_entry_t *) v1;
   const pot_orig_line_entry_t *t2 = (const pot_orig_line_entry_t *) v2;
 
-  return t1->line_pos > t2->line_pos;
+  if (t1->line_pos > t2->line_pos) return 1;
+  if (t1->line_pos < t2->line_pos) return -1;
+
+  return 0;
 }
 
 // the problem with the GNU tdestroy () function is that it doesn't work with mingw etc
@@ -104,25 +107,26 @@ void pot_tree_destroy (pot_tree_entry_t *tree)
 int potfile_init (hashcat_ctx_t *hashcat_ctx)
 {
   const folder_config_t *folder_config = hashcat_ctx->folder_config;
+  const user_options_t  *user_options  = hashcat_ctx->user_options;
   const hashconfig_t    *hashconfig    = hashcat_ctx->hashconfig;
         potfile_ctx_t   *potfile_ctx   = hashcat_ctx->potfile_ctx;
-  const user_options_t  *user_options  = hashcat_ctx->user_options;
 
   potfile_ctx->enabled = false;
 
-  if (user_options->benchmark       == true) return 0;
-  if (user_options->hash_info       == true) return 0;
-  if (user_options->keyspace        == true) return 0;
-  if (user_options->backend_info    == true) return 0;
-  if (user_options->stdout_flag     == true) return 0;
-  if (user_options->speed_only      == true) return 0;
-  if (user_options->progress_only   == true) return 0;
-  if (user_options->usage           == true) return 0;
-  if (user_options->version         == true) return 0;
-  if (user_options->identify        == true) return 0;
-  if (user_options->potfile_disable == true) return 0;
+  if (user_options->usage            > 0)     return 0;
+  if (user_options->backend_info     > 0)     return 0;
+  if (user_options->hash_info        > 0)     return 0;
 
-  if (hashconfig->potfile_disable == true) return 0;
+  if (user_options->benchmark       == true)  return 0;
+  if (user_options->keyspace        == true)  return 0;
+  if (user_options->stdout_flag     == true)  return 0;
+  if (user_options->speed_only      == true)  return 0;
+  if (user_options->progress_only   == true)  return 0;
+  if (user_options->version         == true)  return 0;
+  if (user_options->identify        == true)  return 0;
+  if (user_options->potfile         == false) return 0;
+
+  if (hashconfig->potfile_disable   == true)  return 0;
 
   potfile_ctx->enabled = true;
 
@@ -186,7 +190,7 @@ void potfile_destroy (hashcat_ctx_t *hashcat_ctx)
 int potfile_read_open (hashcat_ctx_t *hashcat_ctx)
 {
   const hashconfig_t  *hashconfig  = hashcat_ctx->hashconfig;
-  potfile_ctx_t       *potfile_ctx = hashcat_ctx->potfile_ctx;
+        potfile_ctx_t *potfile_ctx = hashcat_ctx->potfile_ctx;
 
   if (potfile_ctx->enabled == false) return 0;
 
@@ -205,7 +209,7 @@ int potfile_read_open (hashcat_ctx_t *hashcat_ctx)
 void potfile_read_close (hashcat_ctx_t *hashcat_ctx)
 {
   const hashconfig_t  *hashconfig  = hashcat_ctx->hashconfig;
-  potfile_ctx_t       *potfile_ctx = hashcat_ctx->potfile_ctx;
+        potfile_ctx_t *potfile_ctx = hashcat_ctx->potfile_ctx;
 
   if (potfile_ctx->enabled == false) return;
 
@@ -217,7 +221,7 @@ void potfile_read_close (hashcat_ctx_t *hashcat_ctx)
 int potfile_write_open (hashcat_ctx_t *hashcat_ctx)
 {
   const hashconfig_t  *hashconfig  = hashcat_ctx->hashconfig;
-  potfile_ctx_t       *potfile_ctx = hashcat_ctx->potfile_ctx;
+        potfile_ctx_t *potfile_ctx = hashcat_ctx->potfile_ctx;
 
   if (potfile_ctx->enabled == false) return 0;
 
@@ -236,7 +240,7 @@ int potfile_write_open (hashcat_ctx_t *hashcat_ctx)
 void potfile_write_close (hashcat_ctx_t *hashcat_ctx)
 {
   const hashconfig_t  *hashconfig  = hashcat_ctx->hashconfig;
-  potfile_ctx_t       *potfile_ctx = hashcat_ctx->potfile_ctx;
+        potfile_ctx_t *potfile_ctx = hashcat_ctx->potfile_ctx;
 
   if (potfile_ctx->enabled == false) return;
 
@@ -248,8 +252,7 @@ void potfile_write_close (hashcat_ctx_t *hashcat_ctx)
 void potfile_write_append (hashcat_ctx_t *hashcat_ctx, const char *out_buf, const int out_len, u8 *plain_ptr, unsigned int plain_len)
 {
   const hashconfig_t   *hashconfig   = hashcat_ctx->hashconfig;
-  const user_options_t *user_options = hashcat_ctx->user_options;
-  potfile_ctx_t        *potfile_ctx  = hashcat_ctx->potfile_ctx;
+        potfile_ctx_t  *potfile_ctx  = hashcat_ctx->potfile_ctx;
 
   if (potfile_ctx->enabled == false) return;
 
@@ -274,7 +277,7 @@ void potfile_write_append (hashcat_ctx_t *hashcat_ctx, const char *out_buf, cons
   {
     const bool always_ascii = (hashconfig->opts_type & OPTS_TYPE_PT_ALWAYS_ASCII) ? true : false;
 
-    if ((user_options->outfile_autohex == true) && (need_hexify (plain_ptr, plain_len, hashconfig->separator, always_ascii) == true))
+    if (need_hexify (plain_ptr, plain_len, hashconfig->separator, always_ascii) == true)
     {
       tmp_buf[tmp_len++] = '$';
       tmp_buf[tmp_len++] = 'H';
@@ -329,7 +332,7 @@ void potfile_update_hash (hashcat_ctx_t *hashcat_ctx, hash_t *found, char *line_
     found->pw_buf[found->pw_len] = 0;
   }
 
-  found->cracked = 1;
+  found->cracked_pot = 1;
 
   // if enabled, update also the loopback file
 
@@ -378,10 +381,10 @@ void potfile_update_hashes (hashcat_ctx_t *hashcat_ctx, hash_t *hash_buf, char *
 
 int potfile_remove_parse (hashcat_ctx_t *hashcat_ctx)
 {
-  const hashconfig_t   *hashconfig   = hashcat_ctx->hashconfig;
-  const hashes_t       *hashes       = hashcat_ctx->hashes;
-  const module_ctx_t   *module_ctx   = hashcat_ctx->module_ctx;
-  potfile_ctx_t        *potfile_ctx  = hashcat_ctx->potfile_ctx;
+  const hashconfig_t  *hashconfig   = hashcat_ctx->hashconfig;
+  const hashes_t      *hashes       = hashcat_ctx->hashes;
+  const module_ctx_t  *module_ctx   = hashcat_ctx->module_ctx;
+        potfile_ctx_t *potfile_ctx  = hashcat_ctx->potfile_ctx;
 
   if (potfile_ctx->enabled == false) return 0;
 
@@ -396,7 +399,7 @@ int potfile_remove_parse (hashcat_ctx_t *hashcat_ctx)
   hash_t *hashes_buf = hashes->hashes_buf;
   u32     hashes_cnt = hashes->hashes_cnt;
 
-  // no solution for these special hash types (for instane because they use hashfile in output etc)
+  // no solution for these special hash types (for instance because they use hashfile in output etc)
 
   hash_t hash_buf;
 
@@ -425,7 +428,7 @@ int potfile_remove_parse (hashcat_ctx_t *hashcat_ctx)
   }
 
   // we only need this variable in a very specific situation:
-  // whenever we use --username and --show together we want to keep all hashes sorted within a nice structure
+  // whenever we use --username or --dynamic-x and --show together we want to keep all hashes sorted within a nice structure
 
   pot_tree_entry_t *all_hashes_tree  = NULL;
   pot_tree_entry_t *tree_entry_cache = NULL;
@@ -637,10 +640,10 @@ int potfile_handle_show (hashcat_ctx_t *hashcat_ctx)
   hashes_t      *hashes      = hashcat_ctx->hashes;
   potfile_ctx_t *potfile_ctx = hashcat_ctx->potfile_ctx;
 
-  hash_t *hashes_buf  = hashes->hashes_buf;
+  u32     salts_cnt  = hashes->salts_cnt;
+  salt_t *salts_buf  = hashes->salts_buf;
 
-  u32     salts_cnt = hashes->salts_cnt;
-  salt_t *salts_buf = hashes->salts_buf;
+  hash_t *hashes_buf = hashes->hashes_buf;
 
   pot_orig_line_entry_t *final_buf = (pot_orig_line_entry_t *) hccalloc (hashes->hashes_cnt, sizeof (pot_orig_line_entry_t));
   u32                    final_cnt = 0;
@@ -715,6 +718,22 @@ int potfile_handle_show (hashcat_ctx_t *hashcat_ctx)
           username[user_len] = 0;
         }
 
+        // dynamic-x
+        unsigned char *dynamicx_buf = NULL;
+
+        u32 dynamicx_len = 0;
+
+        dynamicx_t *dynamicx = hash1->hash_info->dynamicx;
+
+        if (dynamicx)
+        {
+          dynamicx_buf = (unsigned char *) (dynamicx->dynamicx_buf);
+
+          dynamicx_len = dynamicx->dynamicx_len;
+
+          dynamicx_buf[dynamicx_len] = 0;
+        }
+
         u8 *tmp_buf = potfile_ctx->tmp_buf;
 
         tmp_buf[0] = 0;
@@ -811,6 +830,25 @@ int potfile_handle_show (hashcat_ctx_t *hashcat_ctx)
           }
         }
 
+        // dynamicx
+        unsigned char *dynamicx_buf = NULL;
+
+        u32 dynamicx_len = 0;
+
+        if (hash->hash_info != NULL)
+        {
+          dynamicx_t *dynamicx = hash->hash_info->dynamicx;
+
+          if (dynamicx)
+          {
+            dynamicx_buf = (unsigned char *) (dynamicx->dynamicx_buf);
+
+            dynamicx_len = dynamicx->dynamicx_len;
+
+            dynamicx_buf[dynamicx_len] = 0;
+          }
+        }
+
         u8 *tmp_buf = potfile_ctx->tmp_buf;
 
         tmp_buf[0] = 0;
@@ -879,10 +917,10 @@ int potfile_handle_left (hashcat_ctx_t *hashcat_ctx)
   module_ctx_t  *module_ctx  = hashcat_ctx->module_ctx;
   potfile_ctx_t *potfile_ctx = hashcat_ctx->potfile_ctx;
 
-  hash_t *hashes_buf = hashes->hashes_buf;
+  u32     salts_cnt  = hashes->salts_cnt;
+  salt_t *salts_buf  = hashes->salts_buf;
 
-  u32     salts_cnt = hashes->salts_cnt;
-  salt_t *salts_buf = hashes->salts_buf;
+  hash_t *hashes_buf = hashes->hashes_buf;
 
   pot_orig_line_entry_t *final_buf = (pot_orig_line_entry_t *) hccalloc (hashes->hashes_cnt, sizeof (pot_orig_line_entry_t));
   u32                    final_cnt = 0;
@@ -957,6 +995,22 @@ int potfile_handle_left (hashcat_ctx_t *hashcat_ctx)
           username[user_len] = 0;
         }
 
+        // dynamic-x
+        unsigned char *dynamicx_buf = NULL;
+
+        u32 dynamicx_len = 0;
+
+        dynamicx_t *dynamicx = hash1->hash_info->dynamicx;
+
+        if (dynamicx)
+        {
+          dynamicx_buf = (unsigned char *) (dynamicx->dynamicx_buf);
+
+          dynamicx_len = dynamicx->dynamicx_len;
+
+          dynamicx_buf[dynamicx_len] = 0;
+        }
+
         u8 *tmp_buf = potfile_ctx->tmp_buf;
 
         tmp_buf[0] = 0;
@@ -1042,6 +1096,25 @@ int potfile_handle_left (hashcat_ctx_t *hashcat_ctx)
           }
         }
 
+        // dynamicx
+        unsigned char *dynamicx_buf = NULL;
+
+        u32 dynamicx_len = 0;
+
+        if (hash->hash_info != NULL)
+        {
+          dynamicx_t *dynamicx = hash->hash_info->dynamicx;
+
+          if (dynamicx)
+          {
+            dynamicx_buf = (unsigned char *) (dynamicx->dynamicx_buf);
+
+            dynamicx_len = dynamicx->dynamicx_len;
+
+            dynamicx_buf[dynamicx_len] = 0;
+          }
+        }
+
         const bool print_eol = (hashconfig->opts_type & OPTS_TYPE_BINARY_HASHFILE) == 0;
 
         u8 *tmp_buf = potfile_ctx->tmp_buf;
