@@ -106,6 +106,21 @@ bool class_upper (const u8 c)
   return ((c >= 'A') && (c <= 'Z'));
 }
 
+bool class_lower_hex (const u8 c)
+{
+  return ((c >= '0') && (c <= '9')) || ((c >= 'a') && (c <= 'f'));
+}
+
+bool class_upper_hex (const u8 c)
+{
+  return ((c >= '0') && (c <= '9')) || ((c >= 'A') && (c <= 'F'));
+}
+
+bool class_sym (const u8 c)
+{
+  return ((c == ' ') || ((c >= '!') && (c <= '/')) || ((c >= ':') && (c <= '@')) || ((c >= '[') && (c <= '`')) || ((c >= '{') && (c <= '~')));
+}
+
 bool class_alpha (const u8 c)
 {
   return (class_lower (c) || class_upper (c));
@@ -217,7 +232,7 @@ int generate_random_rule (char rule_buf[RP_RULE_SIZE], const u32 rp_gen_func_min
     }
   }
 
-  return (rule_pos);
+  return rule_pos;
 }
 
 #define INCR_POS if (++rule_pos == rule_len) return (-1)
@@ -463,6 +478,77 @@ int cpu_rule_to_kernel_rule (char *rule_buf, u32 rule_len, kernel_rule_t *rule)
         SET_P1      (rule, rule_buf[rule_pos]);
         break;
 
+      case RULE_OP_CLASS_BASED: // ~
+        switch (rule_buf[rule_pos+1])
+        {
+          case RULE_OP_MANGLE_REPLACE: // ~s?CY
+            SET_NAME  (rule, RULE_OP_MANGLE_REPLACE_CLASS);
+            INCR_POS;
+            INCR_POS;
+            SET_P0    (rule, rule_buf[rule_pos]);
+            SET_P1    (rule, rule_buf[rule_pos]);
+            break;
+
+          case RULE_OP_MANGLE_PURGECHAR: // ~@?C
+            SET_NAME  (rule, RULE_OP_MANGLE_PURGECHAR_CLASS);
+            INCR_POS;
+            INCR_POS;
+            SET_P0    (rule, rule_buf[rule_pos]);
+            break;
+
+          case RULE_OP_MANGLE_TITLE_SEP: // ~e?C
+            SET_NAME  (rule, RULE_OP_MANGLE_TITLE_SEP_CLASS);
+            INCR_POS;
+            INCR_POS;
+            SET_P0    (rule, rule_buf[rule_pos]);
+            break;
+
+          /*
+          case '!': // ~!?C
+            SET_NAME  (rule, RULE_OP_REJECT_CONTAIN_CLASS);
+            INCR_POS;
+            INCR_POS;
+            SET_P0    (rule, rule_buf[rule_pos]);
+            break;
+          case '/': // ~/?C
+            SET_NAME  (rule, RULE_OP_REJECT_NOT_CONTAIN_CLASS);
+            INCR_POS;
+            INCR_POS;
+            SET_P0    (rule, rule_buf[rule_pos]);
+            break;
+          case '(': // ~(?C
+            SET_NAME  (rule, RULE_OP_REJECT_EQUAL_FIRST_CLASS);
+            INCR_POS;
+            INCR_POS;
+            SET_P0    (rule, rule_buf[rule_pos]);
+            break;
+          case '(': // ~)?C
+            SET_NAME  (rule, RULE_OP_REJECT_EQUAL_LAST_CLASS);
+            INCR_POS;
+            INCR_POS;
+            SET_P0    (rule, rule_buf[rule_pos]);
+            break;
+          case '=': // ~=N?C
+            SET_NAME  (rule, RULE_OP_REJECT_EQUAL_AT_CLASS);
+            INCR_POS;
+            SET_P0    (rule, rule_buf[rule_pos]);
+            INCR_POS;
+            SET_P1    (rule, rule_buf[rule_pos]);
+            break;
+          case '%': // ~%N?C
+            SET_NAME  (rule, RULE_OP_REJECT_CONTAINS_CLASS);
+            INCR_POS;
+            SET_P0    (rule, rule_buf[rule_pos]);
+            INCR_POS;
+            SET_P1    (rule, rule_buf[rule_pos]);
+            break;
+          */
+          default:
+            return -1;
+        }
+
+        break;
+
       default:
         return -1;
     }
@@ -692,6 +778,28 @@ int kernel_rule_to_cpu_rule (char *rule_buf, kernel_rule_t *rule)
         GET_P1      (rule);
         break;
 
+      case RULE_OP_MANGLE_REPLACE_CLASS:
+        rule_buf[rule_pos++] = RULE_OP_CLASS_BASED;
+        rule_buf[rule_pos++] = RULE_OP_MANGLE_REPLACE;
+        rule_buf[rule_pos]   = '?';
+        GET_P0 (rule);
+        GET_P1 (rule);
+        break;
+
+      case RULE_OP_MANGLE_PURGECHAR_CLASS:
+        rule_buf[rule_pos++] = RULE_OP_CLASS_BASED;
+        rule_buf[rule_pos++] = RULE_OP_MANGLE_PURGECHAR;
+        rule_buf[rule_pos]   = '?';
+        GET_P0 (rule);
+        break;
+
+      case RULE_OP_MANGLE_TITLE_SEP_CLASS:
+        rule_buf[rule_pos++] = RULE_OP_CLASS_BASED;
+        rule_buf[rule_pos++] = RULE_OP_MANGLE_TITLE_SEP;
+        rule_buf[rule_pos]   = '?';
+        GET_P0 (rule);
+        break;
+
       case 0:
         if (rule_pos == 0) return -1;
         return rule_pos - 1;
@@ -770,6 +878,18 @@ int kernel_rules_load (hashcat_ctx_t *hashcat_ctx, kernel_rule_t **out_buf, u32
     {
       rule_len = (u32) fgetl (&fp, rule_buf, HCBUFSIZ_LARGE);
 
+      if (rule_line == (u32) -1)
+      {
+        event_log_error (hashcat_ctx, "Unsupported number of lines in rule file %s.", rp_file);
+
+        hcfree (all_kernel_rules_cnt);
+        hcfree (all_kernel_rules_buf);
+
+        hcfree (rule_buf);
+
+        return -1;
+      }
+
       rule_line++;
 
       if (rule_len == 0) continue;
@@ -780,7 +900,21 @@ int kernel_rules_load (hashcat_ctx_t *hashcat_ctx, kernel_rule_t **out_buf, u32
       {
         kernel_rules_buf = (kernel_rule_t *) hcrealloc (kernel_rules_buf, kernel_rules_avail * sizeof (kernel_rule_t), INCR_RULES * sizeof (kernel_rule_t));
 
+        const u32 kernel_rules_avail_old = kernel_rules_avail;
+
         kernel_rules_avail += INCR_RULES;
+
+        if (kernel_rules_avail < kernel_rules_avail_old) // u32 overflow
+        {
+          event_log_error (hashcat_ctx, "Unsupported number of rules in rule file %s.", rp_file);
+
+          hcfree (all_kernel_rules_cnt);
+          hcfree (all_kernel_rules_buf);
+
+          hcfree (rule_buf);
+
+          return -1;
+        }
       }
 
       char in[RP_PASSWORD_SIZE];
@@ -793,20 +927,36 @@ int kernel_rules_load (hashcat_ctx_t *hashcat_ctx, kernel_rule_t **out_buf, u32
 
       if (result == -1)
       {
-        event_log_warning (hashcat_ctx, "Skipping invalid or unsupported rule in file %s on line %u: %s", rp_file, rule_line, rule_buf);
-
+        if (user_options->quiet == false)
+        {
+          event_log_warning (hashcat_ctx, "Skipping invalid or unsupported rule in file %s on line %u: %s", rp_file, rule_line, rule_buf);
+        }
         continue;
       }
 
       if (cpu_rule_to_kernel_rule (rule_buf, rule_len, &kernel_rules_buf[kernel_rules_cnt]) == -1)
       {
-        event_log_warning (hashcat_ctx, "Cannot convert rule for use on OpenCL device in file %s on line %u: %s", rp_file, rule_line, rule_buf);
-
+        if (user_options->quiet == false)
+        {
+          event_log_warning (hashcat_ctx, "Cannot convert rule for use on OpenCL device in file %s on line %u: %s", rp_file, rule_line, rule_buf);
+        }
         memset (&kernel_rules_buf[kernel_rules_cnt], 0, sizeof (kernel_rule_t)); // needs to be cleared otherwise we could have some remaining data
 
         continue;
       }
 
+      if (kernel_rules_cnt == (u32) -1)
+      {
+        event_log_error (hashcat_ctx, "Unsupported number of rules in rule file %s.", rp_file);
+
+        hcfree (all_kernel_rules_cnt);
+        hcfree (all_kernel_rules_buf);
+
+        hcfree (rule_buf);
+
+        return -1;
+      }
+
       kernel_rules_cnt++;
     }
 
@@ -830,18 +980,51 @@ int kernel_rules_load (hashcat_ctx_t *hashcat_ctx, kernel_rule_t **out_buf, u32
 
   for (u32 i = 0; i < user_options->rp_files_cnt; i++)
   {
+    const u32 kernel_rules_cnt_old = kernel_rules_cnt;
+
     kernel_rules_cnt *= all_kernel_rules_cnt[i];
 
+    if (kernel_rules_cnt < kernel_rules_cnt_old) // u32 overflow ?
+    {
+      if (all_kernel_rules_cnt[i] > 0) // at least one "valid" rule
+      {
+        event_log_error (hashcat_ctx, "Unsupported number of rules used in rule chaining.");
+
+        hcfree (all_kernel_rules_cnt);
+        hcfree (all_kernel_rules_buf);
+
+        hcfree (rule_buf);
+        hcfree (repeats);
+
+        return -1;
+      }
+    }
+
     repeats[i + 1] = kernel_rules_cnt;
   }
 
   kernel_rule_t *kernel_rules_buf = (kernel_rule_t *) hccalloc (kernel_rules_cnt, sizeof (kernel_rule_t));
 
+  if (kernel_rules_buf == NULL)
+  {
+    event_log_error (hashcat_ctx, "Not enough allocatable memory (RAM) for this ruleset.");
+
+    hcfree (all_kernel_rules_cnt);
+    hcfree (all_kernel_rules_buf);
+
+    hcfree (repeats);
+
+    return -1;
+  }
+
+  u32 invalid_cnt = 0;
+  u32 valid_cnt = 0;
+
   for (u32 i = 0; i < kernel_rules_cnt; i++)
   {
     u32 out_pos = 0;
 
-    kernel_rule_t *out = &kernel_rules_buf[i];
+    kernel_rule_t *out = &kernel_rules_buf[i - invalid_cnt];
 
     for (u32 j = 0; j < user_options->rp_files_cnt; j++)
     {
@@ -854,18 +1037,31 @@ int kernel_rules_load (hashcat_ctx_t *hashcat_ctx, kernel_rule_t **out_buf, u32
       {
         if (out_pos == RULES_MAX - 1)
         {
-          // event_log_warning (hashcat_ctx, "Truncated chaining of rules %d and %d - maximum functions per rule exceeded.", i, in_off);
+          invalid_cnt++;
 
           break;
         }
+        else
+        {
+          valid_cnt++;
+        }
 
         out->cmds[out_pos] = in->cmds[in_pos];
       }
     }
   }
 
+  if (invalid_cnt > 0)
+  {
+    event_log_warning (hashcat_ctx, "Maximum functions per rule exceeded during chaining of rules.");
+    event_log_warning (hashcat_ctx, "Skipped %u rule chains, %u valid chains remain.", invalid_cnt, valid_cnt);
+    event_log_warning (hashcat_ctx, NULL);
+  }
+
   hcfree (repeats);
 
+  kernel_rules_cnt -= invalid_cnt;
+
   hcfree (all_kernel_rules_cnt);
   hcfree (all_kernel_rules_buf);
 
