@@ -8,6 +8,7 @@
 #include "event.h"
 #include "memory.h"
 #include "filehandling.h"
+#include "folder.h"
 #include "shared.h"
 #include "tuningdb.h"
 
@@ -47,62 +48,94 @@ int sort_by_tuning_db_entry (const void *v1, const void *v2)
 
 int tuning_db_init (hashcat_ctx_t *hashcat_ctx)
 {
-  folder_config_t *folder_config  = hashcat_ctx->folder_config;
   tuning_db_t     *tuning_db      = hashcat_ctx->tuning_db;
   user_options_t  *user_options   = hashcat_ctx->user_options;
+  folder_config_t *folder_config  = hashcat_ctx->folder_config;
 
   tuning_db->enabled = false;
 
-  if (user_options->hash_info     == true) return 0;
-  if (user_options->keyspace      == true) return 0;
-  if (user_options->left          == true) return 0;
-  if (user_options->backend_info  == true) return 0;
-  if (user_options->show          == true) return 0;
-  if (user_options->usage         == true) return 0;
-  if (user_options->version       == true) return 0;
-  if (user_options->identify      == true) return 0;
+  if (user_options->usage         > 0)    return 0;
+  if (user_options->backend_info  > 0)    return 0;
+  if (user_options->hash_info     > 0)    return 0;
+
+  if (user_options->keyspace     == true) return 0;
+  if (user_options->left         == true) return 0;
+  if (user_options->show         == true) return 0;
+  if (user_options->version      == true) return 0;
+  if (user_options->identify     == true) return 0;
 
   tuning_db->enabled = true;
 
-  char *tuning_db_file;
+  char *tuning_db_folder = NULL;
 
-  hc_asprintf (&tuning_db_file, "%s/%s", folder_config->shared_dir, TUNING_DB_FILE);
+  hc_asprintf (&tuning_db_folder, "%s/tunings", folder_config->shared_dir);
 
-  HCFILE fp;
+  char **tuning_db_files = scan_directory (tuning_db_folder);
 
-  if (hc_fopen (&fp, tuning_db_file, "rb") == false)
+  hcfree (tuning_db_folder);
+
+  for (int i = 0; tuning_db_files[i] != NULL; i++)
   {
-    event_log_error (hashcat_ctx, "%s: %s", tuning_db_file, strerror (errno));
+    char *tuning_db_file = tuning_db_files[i];
 
-    return -1;
+    const size_t suflen = strlen (TUNING_DB_SUFFIX);
+
+    const size_t dblen = strlen (tuning_db_file);
+
+    if (dblen < suflen)
+    {
+      hcfree (tuning_db_file);
+
+      continue; // make sure to not do any out-of-boundary reads
+    }
+
+    if (memcmp (tuning_db_file + dblen - suflen, TUNING_DB_SUFFIX, suflen) != 0)
+    {
+      hcfree (tuning_db_file);
+
+      continue;
+    }
+
+    HCFILE fp;
+
+    if (hc_fopen (&fp, tuning_db_file, "rb") == false)
+    {
+      event_log_error (hashcat_ctx, "%s: %s", tuning_db_file, strerror (errno));
+
+      for (int j = 0; tuning_db_files[j] != NULL; j++) hcfree (tuning_db_files[j]);
+
+      return -1;
+    }
+
+    hcfree (tuning_db_file);
+
+    int line_num = 0;
+
+    char *buf = (char *) hcmalloc (HCBUFSIZ_LARGE);
+
+    while (!hc_feof (&fp))
+    {
+      char *line_buf = hc_fgets (buf, HCBUFSIZ_LARGE - 1, &fp);
+
+      if (line_buf == NULL) break;
+
+      line_num++;
+
+      const size_t line_len = in_superchop (line_buf);
+
+      if (line_len == 0) continue;
+
+      if (line_buf[0] == '#') continue;
+
+      tuning_db_process_line (hashcat_ctx, line_buf, line_num);
+    }
+
+    hcfree (buf);
+
+    hc_fclose (&fp);
   }
 
-  hcfree (tuning_db_file);
-
-  int line_num = 0;
-
-  char *buf = (char *) hcmalloc (HCBUFSIZ_LARGE);
-
-  while (!hc_feof (&fp))
-  {
-    char *line_buf = hc_fgets (buf, HCBUFSIZ_LARGE - 1, &fp);
-
-    if (line_buf == NULL) break;
-
-    line_num++;
-
-    const size_t line_len = in_superchop (line_buf);
-
-    if (line_len == 0) continue;
-
-    if (line_buf[0] == '#') continue;
-
-    tuning_db_process_line (hashcat_ctx, line_buf, line_num);
-  }
-
-  hcfree (buf);
-
-  hc_fclose (&fp);
+  hcfree (tuning_db_files);
 
   // todo: print loaded 'cnt' message
 
@@ -225,11 +258,11 @@ bool tuning_db_process_line (hashcat_ctx_t *hashcat_ctx, const char *line_buf, c
 
     char *device_name = token_ptr[0];
 
-    int attack_mode      = -1;
-    int hash_mode        = -1;
-    int vector_width     = -1;
-    int kernel_accel     = -1;
-    int kernel_loops     = -1;
+    int hash_mode     = -1;
+    int attack_mode   = -1;
+    int vector_width  = -1;
+    int kernel_accel  = -1;
+    int kernel_loops  = -1;
 
     if (token_ptr[1][0] != '*') attack_mode   = (int) strtol (token_ptr[1], NULL, 10);
     if (token_ptr[2][0] != '*') hash_mode     = (int) strtol (token_ptr[2], NULL, 10);
@@ -346,7 +379,7 @@ bool tuning_db_process_line (hashcat_ctx_t *hashcat_ctx, const char *line_buf, c
   return true;
 }
 
-tuning_db_entry_t *tuning_db_search_real (hashcat_ctx_t *hashcat_ctx, const char *device_name, const cl_device_type device_type, int attack_mode, const int hash_mode)
+static tuning_db_entry_t *tuning_db_search_real (hashcat_ctx_t *hashcat_ctx, const char *device_name, const cl_device_type device_type, int attack_mode, const int hash_mode)
 {
   tuning_db_t *tuning_db = hashcat_ctx->tuning_db;
 
@@ -460,7 +493,7 @@ tuning_db_entry_t *tuning_db_search_real (hashcat_ctx_t *hashcat_ctx, const char
 
 tuning_db_entry_t *tuning_db_search (hashcat_ctx_t *hashcat_ctx, const char *device_name, const cl_device_type device_type, int attack_mode, const int hash_mode)
 {
-  tuning_db_entry_t *entry;
+  tuning_db_entry_t *entry = NULL;
 
   const char *NV_prefix = (const char *) "NVIDIA ";
 
@@ -482,5 +515,7 @@ tuning_db_entry_t *tuning_db_search (hashcat_ctx_t *hashcat_ctx, const char *dev
 
   entry = tuning_db_search_real (hashcat_ctx, device_name, device_type, attack_mode, hash_mode);
 
-  return entry;
+  if (entry) return entry;
+
+  return NULL;
 }
