@@ -13,6 +13,7 @@
 #include "rp_cpu.h"
 #include "shared.h"
 #include "wordlist.h"
+#include "bitops.h"
 #include "emu_inc_hash_sha1.h"
 
 size_t convert_from_hex (hashcat_ctx_t *hashcat_ctx, char *line_buf, const size_t line_len)
@@ -36,7 +37,7 @@ size_t convert_from_hex (hashcat_ctx_t *hashcat_ctx, char *line_buf, const size_
     return (i);
   }
 
-  if (user_options->wordlist_autohex_disable == false)
+  if (user_options->wordlist_autohex == true)
   {
     if (is_hexify ((const u8 *) line_buf, line_len) == true)
     {
@@ -59,6 +60,11 @@ int load_segment (hashcat_ctx_t *hashcat_ctx, HCFILE *fp)
 
   wl_data->cnt = hc_fread (wl_data->buf, 1, wl_data->incr - 1000, fp);
 
+  if (wl_data->cnt == (size_t) -1)
+  {
+    return -1;
+  }
+
   wl_data->buf[wl_data->cnt] = 0;
 
   if (wl_data->cnt == 0) return 0;
@@ -338,7 +344,12 @@ void get_next_word (hashcat_ctx_t *hashcat_ctx, HCFILE *fp, char **out_buf, u32
     return;
   }
 
-  load_segment (hashcat_ctx, fp);
+  if (load_segment (hashcat_ctx, fp) == -1)
+  {
+    event_log_error (hashcat_ctx, "Error reading file!\n");
+
+    return;
+  }
 
   get_next_word (hashcat_ctx, fp, out_buf, out_len);
 }
@@ -481,11 +492,22 @@ int count_words (hashcat_ctx_t *hashcat_ctx, HCFILE *fp, const char *dictfile, u
 
   memcpy (dictfile_padded, dictfile, dictfile_len);
 
+  for (size_t i = 0, j = 0; i < dictfile_len; i += 4, j += 1)
+  {
+    dictfile_padded[j] = byte_swap_32 (dictfile_padded[j]);
+  }
+
   sha1_ctx_t sha1_ctx;
   sha1_init   (&sha1_ctx);
   sha1_update (&sha1_ctx, dictfile_padded, dictfile_len);
   sha1_final  (&sha1_ctx);
 
+  sha1_ctx.h[0] = byte_swap_32 (sha1_ctx.h[0]);
+  sha1_ctx.h[1] = byte_swap_32 (sha1_ctx.h[1]);
+  sha1_ctx.h[2] = byte_swap_32 (sha1_ctx.h[2]);
+  sha1_ctx.h[3] = byte_swap_32 (sha1_ctx.h[3]);
+  sha1_ctx.h[4] = byte_swap_32 (sha1_ctx.h[4]);
+
   hcfree (dictfile_padded);
 
   memcpy (d.hash_filename, sha1_ctx.h, 16);
@@ -500,7 +522,7 @@ int count_words (hashcat_ctx_t *hashcat_ctx, HCFILE *fp, const char *dictfile, u
 
       if (user_options_extra->attack_kern == ATTACK_KERN_STRAIGHT)
       {
-        if (overflow_check_u64_mul (keyspace, straight_ctx->kernel_rules_cnt) == false) return -1;
+        if (overflow_check_u64_mul (keyspace, straight_ctx->kernel_rules_cnt) == true) return -1;
 
         keyspace *= straight_ctx->kernel_rules_cnt;
       }
@@ -508,13 +530,13 @@ int count_words (hashcat_ctx_t *hashcat_ctx, HCFILE *fp, const char *dictfile, u
       {
         if (((hashconfig->opti_type & OPTI_TYPE_OPTIMIZED_KERNEL) == 0) && (user_options->attack_mode == ATTACK_MODE_HYBRID2))
         {
-          if (overflow_check_u64_mul (keyspace, mask_ctx->bfs_cnt) == false) return -1;
+          if (overflow_check_u64_mul (keyspace, mask_ctx->bfs_cnt) == true) return -1;
 
           keyspace *= mask_ctx->bfs_cnt;
         }
         else
         {
-          if (overflow_check_u64_mul (keyspace, combinator_ctx->combs_cnt) == false) return -1;
+          if (overflow_check_u64_mul (keyspace, combinator_ctx->combs_cnt) == true) return -1;
 
           keyspace *= combinator_ctx->combs_cnt;
         }
@@ -548,7 +570,10 @@ int count_words (hashcat_ctx_t *hashcat_ctx, HCFILE *fp, const char *dictfile, u
 
   while (!hc_feof (fp))
   {
-    load_segment (hashcat_ctx, fp);
+    if (load_segment (hashcat_ctx, fp) == -1)
+    {
+      return -2;
+    }
 
     comp += wl_data->cnt;
 
@@ -608,7 +633,7 @@ int count_words (hashcat_ctx_t *hashcat_ctx, HCFILE *fp, const char *dictfile, u
 
       if (user_options_extra->attack_kern == ATTACK_KERN_STRAIGHT)
       {
-        if (overflow_check_u64_add (cnt, straight_ctx->kernel_rules_cnt) == false) return -1;
+        if (overflow_check_u64_add (cnt, straight_ctx->kernel_rules_cnt) == true) return -1;
 
         cnt += straight_ctx->kernel_rules_cnt;
       }
@@ -616,13 +641,13 @@ int count_words (hashcat_ctx_t *hashcat_ctx, HCFILE *fp, const char *dictfile, u
       {
         if (((hashconfig->opti_type & OPTI_TYPE_OPTIMIZED_KERNEL) == 0) && (user_options->attack_mode == ATTACK_MODE_HYBRID2))
         {
-          if (overflow_check_u64_add (cnt, mask_ctx->bfs_cnt) == false) return -1;
+          if (overflow_check_u64_add (cnt, mask_ctx->bfs_cnt) == true) return -1;
 
           cnt += mask_ctx->bfs_cnt;
         }
         else
         {
-          if (overflow_check_u64_add (cnt, combinator_ctx->combs_cnt) == false) return -1;
+          if (overflow_check_u64_add (cnt, combinator_ctx->combs_cnt) == true) return -1;
 
           cnt += combinator_ctx->combs_cnt;
         }
@@ -683,20 +708,21 @@ int wl_data_init (hashcat_ctx_t *hashcat_ctx)
 
   wl_data->enabled = false;
 
-  if (user_options->benchmark    == true) return 0;
-  if (user_options->hash_info    == true) return 0;
-  if (user_options->left         == true) return 0;
-  if (user_options->usage        == true) return 0;
-  if (user_options->version      == true) return 0;
+  if (user_options->usage         > 0)    return 0;
   if (user_options->backend_info  > 0)    return 0;
+  if (user_options->hash_info     > 0)    return 0;
+
+  if (user_options->benchmark    == true) return 0;
+  if (user_options->left         == true) return 0;
+  if (user_options->version      == true) return 0;
 
   wl_data->enabled = true;
 
-  wl_data->buf   = (char *) hcmalloc (user_options->segment_size);
-  wl_data->avail = user_options->segment_size;
-  wl_data->incr  = user_options->segment_size;
-  wl_data->cnt   = 0;
-  wl_data->pos   = 0;
+  wl_data->buf     = (char *) hcmalloc (user_options->segment_size);
+  wl_data->avail   = user_options->segment_size;
+  wl_data->incr    = user_options->segment_size;
+  wl_data->cnt     = 0;
+  wl_data->pos     = 0;
 
   /**
    * choose dictionary parser
@@ -717,13 +743,13 @@ int wl_data_init (hashcat_ctx_t *hashcat_ctx)
     }
     else
     {
-      if (user_options->wordlist_autohex_disable == false)
+      if (user_options->wordlist_autohex == true)
       {
         wl_data->func = get_next_word_lm_hex_or_text; // might be $HEX[] notation
       }
       else
       {
-        wl_data->func = get_next_word_lm_text;        // treat as nromal text
+        wl_data->func = get_next_word_lm_text;        // treat as normal text
       }
     }
   }
