@@ -9,7 +9,7 @@ use strict;
 use warnings;
 
 use Digest::SHA  qw (sha512_hex);
-use MIME::Base64 qw (encode_base64);
+use MIME::Base64 qw (encode_base64 decode_base64);
 
 sub module_constraints { [[0, 256], [0, 256], [0, 55], [0, 55], [0, 55]] }
 
