@@ -11,7 +11,7 @@ use warnings;
 use Digest::MD5 qw (md5);
 use POSIX       qw (ceil);
 
-sub module_constraints { [[-1, -1], [-1, -1], [0, 55], [1, 4], [-1, -1]] }
+sub module_constraints { [[-1, -1], [-1, -1], [0, 47], [1, 4], [0, 48]] }
 
 sub pseudo_base64
 {
