@@ -24,11 +24,11 @@ from pygost import gost34112012512
 key    = b"$salt"
 msg    = b"$word"
 digest = hmac.new (key, msg, gost34112012512).digest ()
-sys.stdout.write (binascii.hexlify (digest[::-1]))
+print (binascii.hexlify (digest[::-1]).decode (), end = "")
 
 END_CODE
 
-  my $digest = `python2 -c '$python_code'`;
+  my $digest = `python3 -c '$python_code'`;
 
   my $hash = sprintf ("%s:%s", $digest, $salt);
 
