@@ -8,10 +8,10 @@
 use strict;
 use warnings;
 
-use Crypt::CBC;
-use Crypt::ECB  qw (encrypt);
 use Digest::MD4 qw (md4);
 use Digest::SHA qw (sha1 hmac_sha1 hmac_sha512);
+use Crypt::CBC;
+use Crypt::ECB;
 use Encode;
 
 sub module_constraints { [[0, 256], [-1, -1], [-1, -1], [-1, -1], [-1, -1]] }
@@ -132,11 +132,11 @@ sub module_generate_hash
 
   if ($context == 1)
   {
-     $user_hash = sha1 (encode ("UTF-16LE", $word_buf));
+    $user_hash = sha1 (encode ("UTF-16LE", $word_buf));
   }
   elsif ($context == 2)
   {
-     $user_hash = md4 (encode ("UTF-16LE", $word_buf));
+    $user_hash = md4 (encode ("UTF-16LE", $word_buf));
   }
 
   $user_derivationKey = hmac_sha1 (encode ("UTF-16LE", $SID . "\x00"), $user_hash);
@@ -396,6 +396,7 @@ sub module_verify_hash
   my $cipher_len       = shift @data;
   my $cipher           = shift @data;
 
+  return unless ($context == 1 || $context == 2);
   return unless (length ($cipher) == $cipher_len);
 
   if ($version == 1)
