@@ -9,7 +9,7 @@ use strict;
 use warnings;
 
 use Crypt::PBKDF2;
-use MIME::Base64 qw (encode_base64 decode_base64);
+use MIME::Base64 qw (encode_base64);
 use Encode;
 
 sub module_constraints { [[0, 256], [128, 128], [-1, -1], [-1, -1], [-1, -1]] }
@@ -24,7 +24,7 @@ sub aes_encrypt
 
   my $python_code = <<'END_CODE';
 
-from CryptoPlus.Cipher import AES
+from CryptoPlus.Cipher import python_AES
 import base64
 
 key1 = base64.b64decode (key_main)
@@ -32,7 +32,7 @@ key2 = base64.b64decode (key_tweak)
 
 key = (key1, key2)
 
-cipher = AES.new (key, AES.MODE_XTS)
+cipher = python_AES.new (key, python_AES.MODE_XTS)
 
 sequence = b"\x01"
 
@@ -67,7 +67,7 @@ sub aes_decrypt
 
   my $python_code = <<'END_CODE';
 
-from CryptoPlus.Cipher import AES
+from CryptoPlus.Cipher import python_AES
 import base64
 
 key1 = base64.b64decode (key_main)
@@ -75,7 +75,7 @@ key2 = base64.b64decode (key_tweak)
 
 key = (key1, key2)
 
-cipher = AES.new (key, AES.MODE_XTS)
+cipher = python_AES.new (key, python_AES.MODE_XTS)
 
 sequence = b"\x01"
 
